<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

/**
 * 認証クッキー名称定義
 *
 */
define('ADMIN_AUTH', 'ysc_admin_auth');

/**
 * プログラムコマンド番号定義
 */
define('PARM_COMMAND_NAME', 'p');
define('CMD_LOGIN_FIRST_DISP',	 '0001');					// パスワード設定画面表示要求
define('CMD_LOGIN_FIRST_REQU',	 '0002');					// パスワード設定保存要求
define('CMD_LOGIN_DISP',		 '0003');					// ログイン画面表示要求
define('CMD_LOGIN_REQU',		 '0004');					// ログイン要求
define('CMD_LOGOUT_REQU',		 '0005');					// ログアウト要求
define('CMD_MENU_DISP',			 '0101');					// メニュー画面表示要求
define('CMD_STATUS_DISP',		 '0201');					// カレンダー状態設定画面表示要求
define('CMD_STATUS_REQU',		 '0202');					// カレンダー状態保存要求
define('CMD_CONFIG_DISP',		 '0301');					// 設定情報画面表示要求
define('CMD_COMMON_REQU', 		 '0302');					// 共通設定保存要求
define('CMD_TABLE_STYLE_REQU',	 '0303');					// テーブル表示スタイル設定保存要求
define('CMD_CELL_STYLE_DISP',	 '0401');					// セル表示スタイル設定画面表示要求
define('CMD_CELL_STYLE_REQU',	 '0402');					// セル表示スタイル保存要求
define('CMD_DELETE_PASTDATA',	 '0501');					// 表示期間を過ぎたデータ・画像ファイル削除要求

/**
 * テンプレートファイル定義
 */
define('TPL_LOGIN_FIRST',	'admin_loginfirst.tpl');		// パスワード設定画面
define('TPL_LOGIN',			'admin_login.tpl');				// ログイン画面
define('TPL_MENU', 			'admin_menu.tpl');				// メニュー画面
define('TPL_STATUS',		'admin_status.tpl');			// カレンダー状態設定画面
define('TPL_STYLE',			'admin_style.tpl');				// 表示スタイル設定画面
define('TPL_CELL_STYLE',	'admin_cellstyle.tpl');			// セル表示スタイル設定画面

/**
 * Smarty 設定定義
 */
if(!defined('SMARTY_SETTING')) {
define('SMARTY_SETTING', 1);
define('SMARTY_CLASS_FILE', dirname(__FILE__) . '/../smarty/Smarty.class.php');
define('SMARTY_LEFT_DELIMITER', '<!--{');
define('SMARTY_RIGHT_DELIMITER', '}-->');
define('SMARTY_TEMPLATE_DIR', dirname(__FILE__) . '/templates');
define('SMARTY_CACHE_DIR', dirname(__FILE__) . '/cashe');
define('SMARTY_CONFIG_DIR', dirname(__FILE__) . '/configs');
define('SMARTY_COMPILE_DIR', dirname(__FILE__) . '/templates_c');
}

/**
 * ファイル名パターン定義
 */
define('REG_DATA_FILE', '/^(19[7-9][0-9]|20[0-2][0-9]|203[0-7])\_(0[1-9]|1[0-2])\.xml$/');
define('REG_IMAGE_FILE', '/^(19[7-9][0-9]|20[0-2][0-9]|203[0-7])\_(0[1-9]|1[0-2])\.(gif|png|jpg|jpeg)$/');

require_once(SMARTY_CLASS_FILE);
require_once(dirname(__FILE__) . '/ysc_imgcalendarmodel.php');
require_once(dirname(__FILE__) . '/ysc_imgcalendarview.php');

/**
 * カレンダー管理クラス
 *
 */
class YSC_CalendarAdmin {

	/**
	 * カレンダーモデルクラス
	 *
	 * @var YSC_CalendarModel
	 */
	var $model = NULL;

	/**
	 * カレンダー表示クラス
	 *
	 * @var Smarty
	 */
	var $view = NULL;

	/**
	 * エラーメッセージ
	 *
	 * @var unknown_type
	 */
	var $errormessage = array();


	function YSC_CalendarAdmin($ConfFile, $DataDir, $Year = 0, $Month = 0) {
		$this->model = new YSC_ImageCalendarModel($ConfFile, $DataDir, $Year, $Month);
		if(!$this->model) {
			trigger_error("システムエラーが発生しました", E_USER_ERROR);
		}
		$this->view = new Smarty();
		if(!$this->view) {
			trigger_error("システムエラーが発生しました", E_USER_ERROR);
		}
		$this->view->left_delimiter = SMARTY_LEFT_DELIMITER;
		$this->view->right_delimiter = SMARTY_RIGHT_DELIMITER;
		$this->view->template_dir = SMARTY_TEMPLATE_DIR;
		$this->view->cache_dir = SMARTY_CACHE_DIR;
		$this->view->config_dir = SMARTY_CONFIG_DIR;
		$this->view->compile_dir = SMARTY_COMPILE_DIR;
	}

	/**
	 * アクション関数
	 *
	 */
	function Action() {
		if($this->IsAuth()){
			// 認証済み処理
			switch ($_REQUEST[PARM_COMMAND_NAME]) {
				case CMD_MENU_DISP :			// メニュー画面表示要求
					$this->DisplayMenu();
					break;
				case CMD_STATUS_DISP :			// カレンダー状態設定画面表示用きゅ
					$this->DisplayStatus();
					break;
				case CMD_STATUS_REQU :			// カレンダー状態保存要求
					$this->CmdStatus();
					break;
				case CMD_CONFIG_DISP :			// 設定情報画面表示要求
					$this->DisplayConfig();
					break;
				case CMD_COMMON_REQU :			// 設定情報 共通データ保存要求
					$this->CmdCommon();
					break;
				case CMD_TABLE_STYLE_REQU :		// 設定情報 テーブル表示スタイル保存要求
					$this->CmdTableStyle();
					break;
				case CMD_CELL_STYLE_DISP :		// 設定情報セル表示スタイル表示要求
					$this->DisplayCellStyle();
					break;
				case CMD_CELL_STYLE_REQU :		// 設定情報 セル表示スタイル保存要求
					$this->CmdCellStyle();
					break;
				case CMD_LOGOUT_REQU :			// ログアウト要求
					// クッキー削除
					setcookie(ADMIN_AUTH, false, time() - 3600);
					$this->errormessage[] = 'ログアウトしました';
					// パスワード入力画面表示
					$this->DisplayLogin();
					break;
				case CMD_DELETE_PASTDATA :		// 表示期間を過ぎたデータ・画像ファイルの削除
					$this->CmdDeletePastData();
					break;
				default:
					$this->DisplayMenu();
					break;
			}
		}
		else {
			switch ($_REQUEST[PARM_COMMAND_NAME]) {
				case CMD_LOGIN_REQU :
					if($this->model->IsExistsPassword()) {
						$this->CmdLogin();
					}
					else {
						$this->errormessage[] = 'パスワードが設定されていません';
						// パスワード未設定の場合
						// パスワード設定画面表示
						$this->DisplayLoginFirst();
					}
					break;
				case CMD_LOGIN_FIRST_REQU :
					$this->CmdLoginFirst();
					break;
				default:
					// 未認証処理
					if($this->model->IsExistsPassword()) {
						// パスワード入力画面表示
						$this->DisplayLogin();
					}
					else {
						// 初回表示時は、パスワード設定画面表示
						$this->DisplayLoginFirst();
					}
					break;
			}
		}
	}

	/**
	 * パスワード設定要求処理
	 *
	 */
	function CmdLoginFirst() {
		if(('' != $_REQUEST['pass1']) && ('' != $_REQUEST['pass2'])){
			if($_REQUEST['pass1'] == $_REQUEST['pass2']) {
				if($this->model->setConfigPassword($_REQUEST['pass1'])) {
					if($this->model->SaveConfig()){
						// 認証クッキー設定
						setcookie(ADMIN_AUTH, true, 0);
						// メニュー画面表示
						$this->DisplayMenu();
						return;
					}
					else {
						trigger_error("設定情報の保存ができません", E_USER_ERROR);
					}
				}
				else {
					$this->errormessage[] = '登録できないパスワード文字列です';
				}
			}
			else {
				$this->errormessage[] = '確認用パスワードと一致していません';
			}
		}
		else {
			$this->errormessage[] = 'パスワードが入力されていません';
		}
		// パスワード設定画面再表示
		$this->DisplayLoginFirst();
	}

	/**
	 * ログイン要求処理
	 *
	 */
	function CmdLogin() {
		if('' != $_REQUEST['pass']) {
			if($this->model->checkPassword($_REQUEST['pass'])) {
				// 認証クッキー設定
				setcookie(ADMIN_AUTH, true, 0);
				// メニュー画面表示
				$this->DisplayMenu();
				return;
			}
			else {
				$this->errormessage[] = 'パスワードが間違っています';
			}
		}
		else {
			$this->errormessage[] = 'パスワードが入力されていません';
		}
		// パスワード入力画面再表示
		$this->DisplayLogin();
	}

	/**
	 * 設定情報 共通データ保存要求
	 *
	 */
	function CmdCommon() {
		// 設定情報 共通データ取得
		$this->model->getConfigCommon($Common);
		if(!isset($Common)) {
			trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
		}
		foreach($Common as $Key => $Value) {
			if(isset($_REQUEST[$Key])){
				$Common[$Key] = $_REQUEST[$Key];
			}
		}

		// 設定情報 共通データ設定
		$this->model->errormessage = array();
		if(!$this->model->setConfigCommon($Common)) {
			$this->errormessage = $this->model->errormessage;
		}
		else {
			// 設定情報保存
			if($this->model->SaveConfig()){
				$this->errormessage[] = '設定情報 共通データを保存しました。';
			}
			else {
				trigger_error("設定情報 共通データが保存できません", E_USER_ERROR);
			}
		}
		// 再表示
		$this->DisplayConfig($Common);
	}

	/**
	 * 設定情報 テーブル表示スタイルデータ保存要求処理
	 *
	 */
	function CmdTableStyle() {
		// 設定情報 テーブル表示スタイルデータ取得
		$this->model->getConfigTableStyles($TableStyle);
		if(!isset($TableStyle)) {
			trigger_error("設定情報 テーブル表示スタイルデータが取得できませんでした", E_USER_ERROR);
		}

		foreach($TableStyle as $Key => $Value) {
			if(isset($_REQUEST[$Key])){
				if(is_array($Value)) {
					if(array_key_exists('value',$Value)) {
						$TableStyle[$Key]['value'] = $_REQUEST[$Key];
					}
				}
				else {
					$TableStyle[$Key] = $_REQUEST[$Key];
				}
			}
		}

		// 設定情報 テーブル表示スタイルデータ設定
		$this->model->errormessage = array();
		if(!$this->model->setConfigTableStyles($TableStyle)) {
			$this->errormessage = $this->model->errormessage;
		}
		else {
			// 設定情報保存
			if($this->model->SaveConfig()){
				$this->errormessage[] = '設定情報 テーブル表示スタイルデータを保存しました。';
			}
			else {
				trigger_error("設定情報 テーブル表示レイアウトデータが保存できません", E_USER_ERROR);
			}
		}
		// 再表示
		$this->DisplayConfig(NULL, $TableStyle);
	}

	/**
	 * 設定情報 セル表示スタイルデータ保存要求処理
	 *
	 */
	function CmdCellStyle() {
		if('' == $_REQUEST['cellstylename']) {
			trigger_error("設定情報 セル表示スタイル名が確認できませんでした", E_USER_ERROR);
		}
		// 設定情報 セル表示スタイルデータ取得
		$this->model->getConfigCellStyle($_REQUEST['cellstylename'], $Cell);
		if(!isset($Cell)) {
			trigger_error("設定情報 セル表示スタイルデータが取得できませんでした", E_USER_ERROR);
		}

		foreach($Cell as $Key => $Value) {
			if(is_array($Value)) {
				if('text' == $Key) {
					foreach($Value as $Text) {
						$Index = $Text['_attr']['id'] - 1;
						foreach($Text as $TKey => $TValue) {
							$Name = 'text_' . $Index . '_' . $TKey;
							if(isset($_REQUEST[$Name])){
								if(is_array($TValue)) {
									if(array_key_exists('value',$TValue)) {
										$Cell['text'][$Index][$TKey]['value'] = $_REQUEST[$Name];
									}
								}
								else {
									$Cell['text'][$Index][$TKey] = $_REQUEST[$Name];
								}
							}
						}
					}
				}
				else {
					if(array_key_exists('value',$Value) && isset($_REQUEST[$Key])) {
						$Cell[$Key]['value'] = $_REQUEST[$Key];
					}
				}
			}
			else {
				if(isset($_REQUEST[$Key])) {
					$Cell[$Key] = $_REQUEST[$Key];
				}
			}
		}

		// 設定情報 セル表示スタイルデータ設定
		$this->model->errormessage = array();
		if(!$this->model->setConfigCellStyle($Cell)) {
			$this->errormessage = $this->model->errormessage;
			// セル表示スタイル設定画面 再表示
			$this->DisplayCellStyle($Cell);
			return;
		}
		else {
			// 設定情報保存
			if($this->model->SaveConfig()){
				$this->errormessage[] = '設定情報 セル表示スタイルデータを保存しました。';
			}
			else {
				trigger_error("設定情報 セル表示レイアウトデータが保存できません", E_USER_ERROR);
			}
		}
		// 設定情報画面表示へ戻る
		$this->DisplayConfig();
	}

	function CmdStatus() {
		if(!preg_match(REG_YEAR, $_REQUEST['year']) ||
		   !preg_match(REG_MONTH, $_REQUEST['month'])) {
			trigger_error("カレンダー年月が確認できませんでした", E_USER_ERROR);
		}

		if(!$this->model->setYearMonth((int)$_REQUEST['year'],
									   (int)$_REQUEST['month'])) {
			trigger_error("カレンダー情報が取得できませんでした", E_USER_ERROR);
		}

		// 新規カレンダー情報を作成する（表示スタイルが変更されている場合がある為）
		$this->model->getData($Data);
		if(!$Data) {
			trigger_error("カレンダー情報が取得できませんでした", E_USER_ERROR);
		}

		// 設定情報 テーブル表示スタイルデータ取得
		$this->model->getConfigTableStyles($TableStyle);
		if(!isset($TableStyle)) {
			trigger_error("設定情報 テーブル表示スタイルデータが取得できませんでした", E_USER_ERROR);
		}
		$TextCnt = (int)$TableStyle['daytext'];
		$Index = 0;
		$DataRef = &$Data['cells']['day']['cell'];
		foreach($DataRef as $Cell) {
			// 日付セルは、ステータス文字列は配列の２番目以降なので添え字１から始める
			for($Cnt = 1; $Cnt <= $TextCnt; ++$Cnt) {
				$Key = 'daycell_' . $Index . '_' . $Cnt;
				if(isset($_REQUEST[$Key])) {
					$DataRef[$Index]['text'][$Cnt]['value'] = $_REQUEST[$Key];
				}
			}
			++$Index;
		}
		// 設定情報 共通データ取得
		$this->model->getConfigCommon($Common);
		if(!isset($Common)) {
			trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
		}
		// 画像保存する設定の場合
		if('0' != $Common['imgoutput']) {
			// 画像カレンダー表示用データ取得
			$this->model->getImageViewData($ImgVewData);
			$objImgView = new YSC_ImageCalendarView($ImgVewData, CAL_IMG_TYPE_GIF);
			if(!$objImgView) {
				trigger_error("システムエラーが発生しました", E_USER_ERROR);
			}
			// ディレクトリ名にセパレータが付加されている場合は、取り除く
			$Dir = $Common['imgoutputdir'];
			if(substr($Dir, -1) == '/'){
				$Dir = substr($Dir, 0, strlen($Dir) - 1);
			}
			if(file_exists($Dir)) {
				$objImgView->getImageType($Ext);
				$File = sprintf("/%04d_%02d", $Data['year'], $Data['month']) . '.' . $Ext;
				$Data['imgfile'] = $Dir . $File;
			}
			else {
					$this->errormessage[] = '画像保存先ディレクトリ指定が不正です';
			}
		}
		$this->model->errormessage = array();
		if(!$this->model->setData($Data)) {
			$this->errormessage = $this->model->errormessage;
			if('0' != $Common['imgoutput']) {
				$Data['imgfile'] = '';
			}
		}
		else {
			// カレンダー情報保存
			if($this->model->SaveData()) {
				$this->errormessage[] = 'カレンダー情報を保存しました';
				// 画像保存する設定の場合
				if('0' != $Common['imgoutput'] && '' != $Data['imgfile']) {
					// 画像カレンダー表示用データ取得
					$this->model->getImageViewData($ImgVewData);
					$objImgView = new YSC_ImageCalendarView($ImgVewData, CAL_IMG_TYPE_GIF);
					if(!$objImgView) {
						trigger_error("システムエラーが発生しました", E_USER_ERROR);
					}
					// カレンダー画像保存
					if(!$objImgView->Save()) {
						$this->errormessage[] = 'カレンダー画像が保存できませんでした';
						// 画像ファイルを消去してカレンダー情報再保存
						$Data['imgfile'] = '';
						$this->model->SaveData($Data['imgfile']);
					}
					else {
						$this->errormessage[] = 'カレンダー画像を保存しました';
					}
				}
			}
			else {
				trigger_error("カレンダー情報が保存できません", E_USER_ERROR);
			}
		}

		// カレンダー状態設定画面再表示
		$this->DisplayStatus();
	}

	function CmdDeletePastData() {
		$this->model->getConfigCommon($Common);
		if(!$Common) {
			trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
		}
		YSC_CalendarUtil::monthadd((int)$Common['pre_month'] * -1, $PreYear, $PreMonth);
		$PreTime = mktime(0, 0, 0, $PreMonth, 1, $PreYear);
		for($i = 0; $i < 2; ++$i) {
			if(0 == $i) {
				$this->model->getDataDir($Dir);
				$Reg = REG_DATA_FILE;
			}
			else {
				$Dir = $Common['imgoutputdir'];
				$Reg = REG_IMAGE_FILE;
			}
			if('' == $Dir){
				$Dir = dirname(__FILE__);
			}
			else if(substr($Dir, -1) == '/'){
				$Dir = substr($Dir, 0, strlen($Dir) - 1);
			}
			$Hdl = opendir($Dir);
			if(!$Hdl) {
				break;
			}
			while (($File = readdir($Hdl)) !== false) {
				if(preg_match($Reg, $File)) {
					$FileParts = explode('_', substr($File,0,7));
					$Ftime = mktime(0, 0, 0, (int)$FileParts[1], 1, (int)$FileParts[0]);
					if($PreTime > $Ftime) {
						$DelFile = $Dir . '/' . $File;
						if(YSC_CalendarUtil::filelock($DelFile)) {
							unlink($DelFile);
							YSC_CalendarUtil::fileunlock($DelFile);
						}
					}
				}
			}
		}
		$this->errormessage[] = '表示期間を過ぎたカレンダー情報ファイル・画像カレンダーファイルを削除しました';
		// メニュー画面表示へ戻る
		$this->DisplayMenu();
	}

	/**
	 * パスワード設定画面表示
	 *
	 */
	function DisplayLoginFirst() {
		$this->view->assign('ScriptFile', $_SERVER['PHP_SELF']);
		$this->view->assign('CmdName', PARM_COMMAND_NAME);
		$this->view->assign('CmdNo', CMD_LOGIN_FIRST_REQU);
		$this->view->assign('Message', $this->ErrMsgtoHtml());
		$this->view->assign('ProgCredit', PROG_CREDIT);

		// 画面表示
		$this->view->display(TPL_LOGIN_FIRST);
	}

	/**
	 * ログイン画面表示
	 *
	 */
	function DisplayLogin() {
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);
		$this->view->assign('CmdName', PARM_COMMAND_NAME);
		$this->view->assign('CmdNo', CMD_LOGIN_REQU);
		$this->view->assign('Message', $this->ErrMsgtoHtml());
		$this->view->assign('ProgCredit', PROG_CREDIT);

		// 画面表示
		$this->view->display(TPL_LOGIN);
	}

	/**
	 * メニュー画面表示
	 *
	 */
	function DisplayMenu() {
		$this->view->assign('URLStatus', $this->MakeQueryString(CMD_STATUS_DISP));
		$this->view->assign('URLStyle', $this->MakeQueryString(CMD_CONFIG_DISP));
		$this->view->assign('URLLogout', $this->MakeQueryString(CMD_LOGOUT_REQU));
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);
		$this->view->assign('CmdName', PARM_COMMAND_NAME);
		$this->view->assign('CmdDeleteData', CMD_DELETE_PASTDATA);
		$this->view->assign('Message', $this->ErrMsgtoHtml());
		$this->view->assign('ProgCredit', PROG_CREDIT);

		// 画面表示
		$this->view->display(TPL_MENU);
	}

	/**
	 * カレンダー状態設定画面表示
	 *
	 */
	function DisplayStatus() {
		// 日付指定ありの場合、カレンダー情報読み直し
		if(preg_match(REG_YEAR, $_REQUEST['year']) &&
		   preg_match(REG_MONTH, $_REQUEST['month'])) {
		   if(!$this->model->setYearMonth((int)$_REQUEST['year'],
		   								 (int)$_REQUEST['month'])) {
				trigger_error("カレンダー情報が取得できませんでした", E_USER_ERROR);
		   	}
		}

		// 設定情報 共通データ取得
		$this->model->getConfigCommon($Common);
		if(!isset($Common)) {
			trigger_error("設定情報が取得できませんでした", E_USER_ERROR);
		}

		// 経過日表示を強制的に「表示する」設定にする
		$Common['pastoutput'] = '1';

		if(!$this->model->setConfigCommon($Common)) {
			trigger_error("設定情報の設定ができませんでした", E_USER_ERROR);
		}

		// 表示用データ取得
		$this->model->getHtmlViewData($ViewData);
		if(!$ViewData) {
			trigger_error("カレンダー情報が取得できませんでした", E_USER_ERROR);
		}


		// 状態選択リスト作成
		$StatusList = explode(',', $Common['daytextlist']);
		if('' != $Common['paststring']) {
			if(!in_array($Common['paststring'], $StatusList)){
				$StatusList[] = $Common['paststring'];
			}
		}
		if('' != $Common['daytextdefault']) {
			if(!in_array($Common['daytextdefault'], $StatusList)){
				$StatusList[] = $Common['daytextdefault'];
			}
		}
		if('' != $Common['regularholidaytext']) {
			if(!in_array($Common['regularholidaytext'], $StatusList)){
				$StatusList[] = $Common['regularholidaytext'];
			}
		}

		$CellStatus = array();
		$DIndex = 0;
		$RegHolidayString = $Common['regularholidaytext'];
		$ReguFlg = $this->model->MakeRegularHolidayList($ReguList);
		foreach($ViewData['daycells'] as $DayCell) {
			$CellStatus[$DIndex] = array(false);
			foreach($DayCell['text'] as $Text) {
				if(1 != (int)$Text['id']) {
					$List = array();
					if(0 != (int)$DayCell['text'][0]['value']) {
						$List = $StatusList;
						$CellStatus[$DIndex][0] = True;
						if($ReguFlg && ('' != $RegHolidayString)) {
							$Week = $DIndex % 7;
							$Day = (int)$DayCell['id'];
							if($this->model->IsRegularHoliday($Week, $Day, $ReguList)) {
								$List = array($RegHolidayString);
							}
						}
						if('' != $Text['value'] && !in_array($Text['value'], $List)) {
							// 選択リスト以外の文字が設定されていたら追加する
							$List[] = $Text['value'];
						}
					}
					$CellStatus[$DIndex][(int)$Text['id'] - 1] = $List;
				}
			}
			++$DIndex;
		}

		$MonthList = array('1','2','3','4','5','6','7','8','9','10','11','12');
		$YearList = array();
		$Year = $ViewData['year'];
		$Month = $ViewData['month'];
		YSC_CalendarUtil::monthadd(-1, $PreYear, $PreMonth, (int)$Year, (int)$Month);
		YSC_CalendarUtil::monthadd( 1, $NextYear, $NextMonth, (int)$Year, (int)$Month);
		YSC_CalendarUtil::monthadd((int)$Common['pre_month'] * -1, $DispPreYear, $DispPreMonth);
		YSC_CalendarUtil::monthadd( (int)$Common['fwd_month'], $DispFwdYear, $DispFwdMonth);
		for($i = -1; $i < 10; ++$i) {
			$YearList[] = (string)($Year + $i);
		}
		$URLList = array(array('year'=>(string)$PreYear, 'month' => (string)$PreMonth),
						 array('year'=>(string)$NextYear, 'month' => (string)$NextMonth),);
		$DataNew = !$this->model->IsExistsDataFile($Year, $Month);
		$ExistList =array();
		$DispMonth = (int)$Common['fwd_month'] + (int)$Common['pre_month'] + 1;
		YSC_CalendarUtil::monthadd((int)$Common['pre_month'] * -1, $Year, $Month);
		for($Cnt = 0; $Cnt < $DispMonth; ++$Cnt) {
			if($this->model->IsExistsDataFile($Year, $Month)) {
				$Date = $Year . '年' . $Month . '月';
				$Param = array('year' => (string)$Year, 'month' => (string)$Month);
				$Url = $this->MakeQueryString(CMD_STATUS_DISP, $Param);
				$ExistList[] = array('date' => $Date, 'url' => $Url);
			}
			if(++$Month > 12) {
				++$Year;
				$Month = 1;
			}
		}

		// 表示データ設定
		$this->view->assign('tableclass', $ViewData['tableclass']);
		$this->view->assign('headercells', $ViewData['headercells']);
		$this->view->assign('weekcells', $ViewData['weekcells']);
		$this->view->assign('daycells', $ViewData['daycells']);
		$this->view->assign('yearlist', $YearList);
		$this->view->assign('statuslist', $CellStatus);
		$this->view->assign('monthlist', $MonthList);
		$this->view->assign('year', (string)$ViewData['year']);
		$this->view->assign('month', (string)$ViewData['month']);
		$this->view->assign('disppreyear', (string)$DispPreYear);
		$this->view->assign('disppremonth', (string)$DispPreMonth);
		$this->view->assign('dispfwdyear', (string)$DispFwdYear);
		$this->view->assign('dispfwdmonth', (string)$DispFwdMonth);
		$this->view->assign('datanew', $DataNew);
		$this->view->assign('datalist', $ExistList);
		$this->view->assign('URLPrevious', $this->MakeQueryString(CMD_STATUS_DISP,$URLList[0]));
		$this->view->assign('URLNext', $this->MakeQueryString(CMD_STATUS_DISP,$URLList[1]));
		$this->view->assign('CmdName', PARM_COMMAND_NAME);
		$this->view->assign('CmdStatusDisp', CMD_STATUS_DISP);
		$this->view->assign('CmdStatusRequ', CMD_STATUS_REQU);
		$this->view->assign('URLMenu', $this->MakeQueryString(CMD_MENU_DISP));
		$this->view->assign('Message', $this->ErrMsgtoHtml());
		$this->view->assign('ProgCredit', PROG_CREDIT);
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);

		// 画面表示
		$this->view->display(TPL_STATUS);
	}

	/**
	 * 設定情報画面表示
	 *
	 * @param array $Common		共通データ
	 * 							NULLの場合は、保持データを表示
	 * @param array $TableStyle	テーブル表示スタイルデータ
	 * 							NULLの場合は、保持データを表示
	 */
	function DisplayConfig($Common = NULL, $TableStyle = NULL) {
		// 設定情報 共通データ取得
		if(is_null($Common)) {
			$this->model->getConfigCommon($Common);
			if(!isset($Common)) {
				trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
			}
		}

		// 設定情報 テーブル表示スタイルデータ取得
		if(is_null($TableStyle)) {
			$this->model->getConfigTableStyles($TableStyle);
			if(!isset($TableStyle)) {
				trigger_error("設定情報 テーブル表示スタイルデータが取得できませんでした", E_USER_ERROR);
			}
		}

		// 設定情報 セル表示スタイルデータ取得
		$this->model->getConfigCellStyles($CellsStyle);
		if(!isset($CellsStyle)) {
			trigger_error("設定情報 セル表示スタイルデータが取得できませんでした", E_USER_ERROR);
		}

		$ONOFFList = array('1', '0');
		$YesNoList = array('する', 'しない');

		// 表示データ設定
		$this->view->assign('common', $Common);
		$this->view->assign('tablestyle', $TableStyle);
		$this->view->assign('cellstyle', $CellsStyle);
		$this->view->assign('onofflist', $ONOFFList);
		$this->view->assign('yesnolist', $YesNoList);
		$this->view->assign('CmdName', PARM_COMMAND_NAME);
		$this->view->assign('CmdCommon', CMD_COMMON_REQU);
		$this->view->assign('CmdTableStyle', CMD_TABLE_STYLE_REQU);
		$this->view->assign('CmdCellsStyle', CMD_CELL_STYLE_DISP);
		$this->view->assign('URLMenu', $this->MakeQueryString(CMD_MENU_DISP));
		$this->view->assign('Message', $this->ErrMsgtoHtml());
		$this->view->assign('ProgCredit', PROG_CREDIT);
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);

		// 画面表示
		$this->view->display(TPL_STYLE);
	}

	/**
	 * セル表示スタイル設定画面表示
	 *
	 * @param array  $Cell				セルスタイルデータ
	 * 									NULLの場合は保持データを表示
	 */
	function DisplayCellStyle($Cell = NULL) {

		// 設定情報 セル表示スタイルデータ取得
		if(is_null($Cell)) {
			if('' != $_REQUEST['stylename']) {
				if(!$this->model->SearchConfigStyleCellByName($_REQUEST['stylename'], $Cell)) {
					trigger_error("設定情報 セル表示スタイルデータが取得できませんでした", E_USER_ERROR);
				}
			}
			else {
				$this->errormessage[] = 'パラメータに誤りがあります';
				// 設定情報 画面再表示
				$this->DisplayConfig();
				return;
			}
		}

		$TextCnt = count($Cell['text']);

		// 表示データ設定
		$this->view->assign('cell', $Cell);
		$this->view->assign('textcount', $TextCnt);
		$this->view->assign('CmdName', PARM_COMMAND_NAME);
		$this->view->assign('CmdNo', CMD_CELL_STYLE_REQU);
		$this->view->assign('URLStyle', $this->MakeQueryString(CMD_CONFIG_DISP));
		$this->view->assign('Message', $this->ErrMsgtoHtml());
		$this->view->assign('ProgCredit', PROG_CREDIT);
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);

		// 画面表示
		$this->view->display(TPL_CELL_STYLE);
	}

	/**
	 * エラーメッセージをHTMLリストに変換する
	 *
	 * @return string HTMLリスト文字列
	 */
	function ErrMsgtoHtml() {
		if(0 == count($this->errormessage)){
			return '';
		}
		$Msg = "<ul>\n";
		foreach($this->errormessage as $ErrMsg) {
			$Msg .= "<li>" . $ErrMsg . "</li>\n";
		}
		$Msg .= "</ul>\n";
		return $Msg;
	}

	/**
	 * クエリ文字列の作成
	 *
	 * @param string $CmdNo				プログラム実行番号
	 * @param array $ParamList			パラメータリストの連想配列
	 * @return string					クエリ文字列
	 */
	function MakeQueryString($CmdNo, $ParamList = NULL) {
		$Query = $_SERVER['PHP_SELF'] . '?' . PARM_COMMAND_NAME . '=' . $CmdNo;
		if(!is_null($ParamList)){
			foreach($ParamList as $Name => $Value) {
				$Query .= '&' . $Name . '=' . $Value;
			}
		}
		return $Query;
	}

	/**
	 * ログイン認証クッキーの設定
	 * @return unknown_type
	 */
	function setAuthCookei() {
		return setcookie(ADMIN_AUTH, true);
	}

	/**
	 * ログイン認証状態の判定
	 *
	 * @return true / false : 認証済 / 未認証
	 */
	function IsAuth() {
		if(isset($_COOKIE[ADMIN_AUTH])){
			return true;
		}
		return false;
	}
}

?>