<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

/**
 * Smarty 設定定義
 *
 */
if(!defined('SMARTY_SETTING')) {
define('SMARTY_SETTING', 1);
define('SMARTY_CLASS_FILE', dirname(__FILE__) . '/../smarty/Smarty.class.php');
define('SMARTY_LEFT_DELIMITER', '<!--{');
define('SMARTY_RIGHT_DELIMITER', '}-->');
define('SMARTY_TEMPLATE_DIR', dirname(__FILE__) . '/../templates');
define('SMARTY_CACHE_DIR', dirname(__FILE__) . '/../cashe');
define('SMARTY_CONFIG_DIR', dirname(__FILE__) . '/../configs');
define('SMARTY_COMPILE_DIR', dirname(__FILE__) . '/../templates_c');
}
require_once(SMARTY_CLASS_FILE);
require_once(dirname(__FILE__) . '/ysc_calendarmodel.php');
require_once(dirname(__FILE__) . '/ysc_calendarutil.php');

/**
 * HTMLカレンダーテンプレートファイル
 */
define('TPL_CALENDAR', 'calendar.tpl');
define('TPL_CALENDAR_ENCODE', 'UTF-8');


/**
 * カレンダークラス
 *
 */
class YSC_Calendar {

	/**
	 * カレンダークラス
	 *
	 * @var YSC_Calendar
	 */
	var $model = NULL;

	/**
	 * スマーティクラス
	 *
	 * @var Smarty
	 */
	var $view = NULL;

	/**
	 * 表示用オプションデータ
	 *
	 * @var array(array('Key'=>$Value))
	 */
	var $viewoption = array();

	/**
	 * コンストラクタ
	 *
	 * @param string $ConfFile			設定情報ファイル
	 * @param string_type $DataDir		カレンダー情報格納ディレクトリ
	 * @param int $Year					表示カレンダー年
	 * @param int $Month				表示カレンダー月
	 */
	function YSC_Calendar($ConfFile, $DataDir, $Year = 0, $Month = 0) {
		$this->model = new YSC_CalendarModel($ConfFile, $DataDir, $Year, $Month);
		if(!$this->model) {
			trigger_error("システムエラーが発生しました", E_USER_ERROR);
		}
		$this->view = new Smarty();
		if(!($this->view)) {
			trigger_error("システムエラーが発生しました", E_USER_ERROR);
		}
		$this->view->left_delimiter = SMARTY_LEFT_DELIMITER;
		$this->view->right_delimiter = SMARTY_RIGHT_DELIMITER;
		$this->view->template_dir = SMARTY_TEMPLATE_DIR;
		$this->view->cache_dir = SMARTY_CACHE_DIR;
		$this->view->config_dir = SMARTY_CONFIG_DIR;
		$this->view->compile_dir = SMARTY_COMPILE_DIR;
	}

	/**
	 * 表示オプションデータの設定
	 *
	 * @param array $options		追加オプションデータ配列
	 * 								array(array('Key'=>$Value))
	 */
	function setViewOptions(&$options) {
		$this->viewoption = $options;
	}

	/**
	 * 表示オプションデータの全消去
	 *
	 */
	function clearViewOptions() {
		$this->viewoption = array();
	}

	/**
	 * 設定情報ファイルの設定
	 *
	 * @param stirng $File			ファイル名
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigFile($File) {
		return $this->model->setConfigFile($File);
	}

	/**
	 * 設定情報ファイル名の取得
	 *
	 * @param string $File (out)	ファイル名
	 */
	function getConfigFile(&$File) {
		$this->model->getConfigFile($File);
	}

	/**
	 * カレンダー情報保存ディレクトリの設定
	 *
	 * @param string $Dir			ディレクトリ名
	 * @return true / false : 設定成功 / 設定失敗
	 * 	 */
	function setDataDir($Dir) {
		return $this->model->setDataDir($Dir);
	}

	/**
	 * カレンダー情報保存ディレクトリの取得
	 *
	 * @param string $Dir (out)		ディレクトリ名
	 */
	function getDataDir(&$Dir) {
		$this->model->setDataDir($Dir);
	}

	/**
	 * カレンダー情報年月の設定
	 *
	 * @param int $Year				年
	 * @param int $Month			月
	 * @return true / false : 設定成功 / 年月指定が不正
	 */
	function setYearMonth($Year, $Month) {
		return $this->model->setYearMonth($Year, $Month);
	}

	/**
	 * カレンダー情報年月の取得
	 *
	 * @param int $Year (out)		年
	 * @param int $Month (out)		月
	 */
	function getYearMonth(&$Year, &$Month) {
		$this->model->getYearMonth($Year, $Month);
	}

	/**
	 * アクション関数
	 *
	 */
	function Action() {
		// 表示用データ取得
		$this->model->getHtmlViewData($ViewData);
		if(!$ViewData) {
			trigger_error("表示用データが取得できませんでした", E_USER_ERROR);
		}

		$this->model->getConfigCommon($Common);
		if(!$Common) {
			trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
		}
		$this->model->getYearMonth($CurYear, $CurMonth);
		$ListCnt = (int)$Common['pre_month'] + (int)$Common['fwd_month'] + 1;
		$DispLimitString = array();
		$DispLimitOffset = array();
		$Offset = (int)$Common['pre_month'] * -1;
		$Index = 0;
		for($Cnt = 0; $Cnt < $ListCnt; ++$Cnt, ++$Offset){
			YSC_CalendarUtil::monthadd($Offset, $Year, $Month);
			$DispLimitString[$Cnt] = mb_convert_encoding($Year . '年' . $Month . '月', TPL_CALENDAR_ENCODE, "UTF-8");
			$DispLimitOffset[$Cnt] = $Offset;
			if(($CurYear == $Year) && ($CurMonth == $Month)) {
				$Index = $Offset;
			}
		}

		// 表示用パラメータ設定
		$this->view->assign('year', $ViewData['year']);
		$this->view->assign('month', $ViewData['month']);
		$this->view->assign('tableclass', $ViewData['tableclass']);
		$this->view->assign('headercells', $ViewData['headercells']);
		$this->view->assign('weekcells', $ViewData['weekcells']);
		$this->view->assign('daycells', $ViewData['daycells']);
		$this->view->assign('displimitstring', $DispLimitString);
		$this->view->assign('displimitoffset', $DispLimitOffset);

		$this->view->assign('displimitindex', $Index);
		if($Index > ((int)$Common['pre_month'] * -1)) {
			$this->view->assign('preoffset', $Index - 1);
		}
		if($Index < (int)$Common['fwd_month']) {
			$this->view->assign('fwdoffset', $Index + 1);
		}
		if(isset($this->viewoption)) {
			$this->view->assign('options', $this->viewoption);
		}
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);
		$this->view->assign('ProgCredit', PROG_CREDIT);

		// 表示
		$this->view->display(TPL_CALENDAR);
	}
}

?>