<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

require_once('XML/Serializer.php');
require_once('XML/Unserializer.php');
require_once(dirname(__FILE__) . '/ysc_calendarutil.php');

/**
 * セルスタイル デフォルト名称定義
 *
 */
define('STYLE_HEADER_DEFAULT', 'header_default');		// ヘッダーセル デフォルト表示スタイル名
define('STYLE_WEEK_DEFAULT', 'week_default');			// 曜日セル デ フォルト表示スタイル名
define('STYLE_WEEK_HOLIDAY', 'week_holiday');			// 曜日セル 休日表示スタイル名
define('STYLE_WEEK_WEEKEND', 'week_weekend');			// 曜日セル 週末表示スタイル名
define('STYLE_DAY_DEFAULT', 'day_default');				// 日付セル デ フォルト表示スタイル名
define('STYLE_DAY_HOLIDAY', 'day_holiday');				// 日付セル 休日表示スタイル名
define('STYLE_DAY_WEEKEND', 'day_weekend');				// 日付セル 週末表示スタイル名
define('STYLE_DAY_TODAY', 'day_today');					// 日付セル 本日表示スタイル名
define('STYLE_DAY_REGHOLIDAY', 'day_regholiday');		// 日付セル 定休日表示スタイル名
define('STYLE_DAY_CUSTOM', 'day_custom_');				// 日付セル カスタム表示スタイル名

/**
 * データ検証用パターン定義
 *
 */
define('REG_PASSWORD', '/^\w{3,8}$/');					// パスワード パターン文字列 (3-8文字英数字)
define('REG_ONOFF', '/^(0|1)$/');						// ON/OFF(1/0)チェック パターン文字列
define('REG_NUMERIC', '/^[-+]?[0-9]{1,}$/');			// 数字チェック パターン文字列
define('REG_HEXCOLOR', '/^[0-9a-fA-F]{6}$/');			// HEXカラーチェック パターン文字列
define('REG_CELLTARGET', '/^(header|week|day)$/');		// セルターゲット属性チェック パターン文字列
define('REG_MUST', '/^[^&<>\"\']{1,128}$/');			// 必須入力チェック パターン文字列
define('REG_ANY', '/^[^&<>\"\']{0,128}$/');				// 任意入力チェック パターン文字列
define('REG_NULL', '//');								// 未チェック パターン文字列
define('REG_FILE', '/^[^\?\*\:\|\"\<\>]*$/');			// ファイル名 パターン文字列
define('REG_WEEK', '/^([^&<>\"\']+\,){6}[^&<>\"\']+$/');// 曜日リスト パターン文字列
define('REG_NUM1', '/^\d{1,3}$/');						// 数字３桁 パターン文字列
define('REG_NUM2', '/^[-+]?\d{1,3}$/');					// 符号付き数字３桁 パターン文字列
define('REG_DATE', '/^\d{0,2}$/');						// 日付文字列 パターン文字列
if(!defined('REG_YEAR')) {
// 年チェック パターン文字列(1970-2037)
define('REG_YEAR', '/^(19[7-9][0-9]|20[0-2][0-9]|203[0-7])$/');
}
if(!defined('REG_MONTH')) {
define('REG_MONTH', '/^([1-9]|1[0-2])$/');				// 月チェック パターン文字列
}
// デモ用使用定義
define('USE_DEMO', 0);

/**
 * カレンダーモデルクラス
 *
 */
class YSC_CalendarModel {

	/**
	 * 設定情報ファイル名
	 *
	 * @var string
	 */
	var $configfile = '';

	/**
	 * カレンダーデータディレクトリ
	 *
	 * @var string
	 */
	var $datadir = '';

	/**
	 * カレントカレンダー情報 年
	 *
	 * @var int
	 */
	var $cur_year = 0;

	/**
	 * カレントカレンダー情報 月
	 *
	 * @var int
	 */
	var $cur_month = 0;

	/**
	 * 設定情報 （配列）
	 *
	 * @var array
	 */
	var $configdata = NULL;

	/**
	 * カレンダー情報 （配列）
	 *
	 * @var array
	 */
	var $calendardata = NULL;

	/**
	 * ラストエラーメッセージ
	 *
	 * @var string
	 */
	var $errormessage = array();


	/**
	 * コンストラクタ
	 *
	 * @param string $ConfigFile	設定情報ファイル
	 * @param string $DataDir		カレンダー情報保存ディレクトリ
	 * @param int $Year				カレンダー 年
	 * @param int $Month			カレンダー月
	 */
	function YSC_CalendarModel($ConfigFile, $DataDir, $Year = 0, $Month = 0) {

		if(!$this->setConfigFile($ConfigFile)){
			trigger_error($this->errormessage[0], E_USER_ERROR);
		}

		if(!$this->setDataDir($DataDir)){
			trigger_error($this->errormessage[0], E_USER_ERROR);
		}

		if(($Year == 0) && ($Month == 0)){
			$Year = date("Y");
			$Month = date("n");
		}

		if(!$this->setYearMonth($Year, $Month)){
			trigger_error($this->errormessage[0], E_USER_ERROR);
		}

	}

	/**
	 * 設定情報ファイルの設定
	 *
	 * @param stirng $File			ファイル名
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigFile($File) {
		// ファイル・ディレクトリ存在チェック
		if(!file_exists($File)){
			$this->errormessage[] = "設定情報ファイルが見つかりません";
			return false;
		}
		if(($this->configfile == '') || ($this->configfile != $File)) {
			if(!$this->LoadXmlFile($File, $this->configdata)) {
				unset($this->configdata);
				$this->errormessage[] = "設定情報ファイルのフォーマットが不正です";
				return false;
			}
		}
		$this->configfile = $File;
		return true;
	}

	/**
	 * 設定情報ファイル名の取得
	 *
	 * @param string $File (out)	ファイル名
	 */
	function getConfigFile(&$File) {
		$File = $this->configfile;
	}

	/**
	 * カレンダー情報保存ディレクトリの設定
	 *
	 * @param string $Dir			ディレクトリ名
	 * @return true / false : 設定成功 / 設定失敗
	 * 	 */
	function setDataDir($Dir) {
		// ディレクトリ名にセパレータが付加されている場合は、取り除く
		if(substr($Dir, -1) == '/'){
			$Dir = substr($Dir, 0, strlen($Dir) - 1);
		}
		// ファイル・ディレクトリ存在チェック
		if(!file_exists($Dir)){
			$this->errormessage[] = "データディレクトリが見つかりません";
			return false;
		}
		$this->datadir = $Dir;
		return true;
	}

	/**
	 * カレンダー情報保存ディレクトリの取得
	 *
	 * @param string $Dir (out)		ディレクトリ名
	 */
	function getDataDir(&$Dir) {
		$Dir = $this->datadir;
	}

	/**
	 * カレンダー情報年月の設定
	 *
	 * @param int $Year				年
	 * @param int $Month			月
	 * @return true / false : 設定成功 / 年月指定が不正
	 */
	function setYearMonth($Year, $Month) {
		if(!preg_match(REG_YEAR, (string)$Year) || !preg_match(REG_MONTH, (string)$Month)) {
			$this->errormessage[] = "カレンダー年月指定が不正です";
		}

		// 保持しているカレンダーデータと年月が変わった場合は、カレンダーデータ再読込
		if(($this->cur_year <> $Year) || ($this->cur_month <> $Month)) {
			$this->LoadCalendarData($Year, $Month);
		}
		$this->cur_year = $Year;
		$this->cur_month = $Month;

		return true;
	}

	/**
	 * カレンダー情報年月の取得
	 *
	 * @param int $Year (out)		年
	 * @param int $Month (out)		月
	 */
	function getYearMonth(&$Year, &$Month) {
		$Year = $this->cur_year;
		$Month = $this->cur_month;
	}

	/**
	 * 設定情報 パスワードの設定
	 *
	 * @param string $Vals 			パスワード文字列
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigPassword(&$Val){
		if(!preg_match(REG_PASSWORD, $Val)){
			return false;
		}
		$this->configdata['password'] = crypt($Val);
		return true;
	}

	/**
	 * 設定情報 パスワード照合
	 *
	 * @param string $Val			照合パスワード文字列
	 */
	function checkPassword(&$Val){
		if(!preg_match(REG_PASSWORD, $Val)){
			return false;
		}
		else {
			$CryptPass = crypt($Val, $this->configdata['password']);
			if($CryptPass == $this->configdata['password'])
				return true;
			else
				return false;
		}
	}

	/**
	 * 設定情報 パスワード設定確認
	 *
	 * @param true /false : 設定済み / 未設定
	 */
	function IsExistsPassword(){
		if('' != $this->configdata['password'])
			return true;
		else
			return false;
	}

	/**
	 * 設定情報 共通データの設定
	 *
	 * @param array $Vals 			設定情報 共通データ
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigCommon(&$Vals){
		if(!$this->IsVaildConfigCommon($Vals)){
			return false;
		}
		$this->configdata['common'] = $Vals;
		return true;
	}

	/**
	 * 設定情報 共通データの取得
	 *
	 * @param array $Vals (out)		共通設定
	 */
	function getConfigCommon(&$Vals){
		$Vals = $this->configdata['common'];
	}

	/**
	 * 設定情報 カレンダーテーブル表示スタイルの設定
	 *
	 * @param array $Vals 			表示スタイル
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigTableStyles($Vals){
		if(!$this->IsVaildConfigTableStyle($Vals)){
			$this->errormessage[] = "設定情報 カレンダーテーブル表示スタイルデータが不正です";
			return false;
		}
		$this->configdata['tablestyle'] = $Vals;
		return true;
	}

	/**
	 * 設定情報 カレンダーテーブル表示スタイルの取得
	 *
	 * @param array $Vals (out)		表示枠設定
	 */
	function getConfigTableStyles(&$Vals){
		$Vals = $this->configdata['tablestyle'];
	}

	/**
	 * 設定情報 セル表示スタイルデータの設定
	 *
	 * @param array $Vals 			セルデータ
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigCellStyle(&$Vals){
		if(!$this->IsVaildConfigCell($Vals)){
			return false;
		}
		$Index = 0;
		foreach($this->configdata['cellstyle']['cell'] as $Cell){
			if($Cell['_attr']['name'] == $Vals['_attr']['name']){
				$this->configdata['cellstyle']['cell'][$Index] = $Vals;
				return true;
			}
			++$Index;
		}
		// 見つからない場合は、ここで追加
		$this->configdata['cellstyle']['cell'][] = $Vals;
		return true;
	}

	/**
	 * 設定情報 セル表示スタイルの一括取得
	 *
	 * @param array $Vals (out)		セル表示スタイル配列
	 */
	function getConfigCellStyles(&$Vals){
		$Vals =  $this->configdata['cellstyle']['cell'];
	}

	/**
	 * 設定情報 ヘッダーセル表示スタイルの取得
	 *
	 * @param string $CellName		セル表示スタイル名
	 * @param array $Vals (out)		ヘッダーセル表示スタイル配列
	 * @return true / false : セルデータ有り / セルデータ無し
	 * 	 */
	function getConfigCellStyle($CellName, &$Vals){
		return  $this->SearchConfigStyleCellByName($CellName, $Vals);
	}

	/**
	 * 設定情報 曜日セル表示スタイルの取得
	 *
	 * @param array $Vals (out)		曜日セル表示スタイル配列
	 */
	function getConfigCellStylesWeek(&$Vals){
		$this->SearchConfigStyleCell('week', $Vals);
	}

	/**
	 * 設定情報 日付セル表示スタイルの取得
	 *
	 * @param array $Vals (out)		日付セル表示スタイル配列
	 */
	function getConfigCellStylesDay(&$Vals){
		$this->SearchConfigStyleCell('day', $Vals);
	}

	/**
	 * カレンダー情報の一括設定
	 *
	 * @param array $Data			カレンダーデータ
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setData(&$Data){
		if(!$this->IsVaildData($Data)) {
			return false;
		}

		$this->calendardata = $Data;
		$this->cur_year = (int)$Data['year'];
		$this->cur_month = (int)$Data['month'];
		return true;
	}

	/**
	 * カレンダー情報の一括取得
	 *
	 * @param array $Data (out)		カレンダーデータ
	 */
	function getData(&$Data){
		$Data = $this->calendardata;
	}

	/**
	 * カレンダー情報 セルデータの設定
	 *
	 * @param string $Target		ターゲットセル( header|week|day)
	 * @param array $Cell			セルデータ
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setDataCell($Target, $Cell) {
		if(!preg_match(REG_TARGET, $Target)){
			$this->errormessage[] = "ターゲットセル指定が不正です";
			return;
		}
		if(!$this->IsVaildDataCell($Target, $Cell)){
			$this->errormessage[] = "カレンダー情報 セルデータが不正です";
			return false;
		}
		$TargetData = &$this->calendardata['cells'][$Target]['cell'];
		if(($Cell['_attr']['style'] == STYLE_DAY_TODAY)) {
			$Day = (int)$Cell['_attr']['id'];
			$Today = mktime(0,0,0, $this->cur_month, $Day, $this->cur_year);
			if('' != YSC_CalendarUtil::ktHolidayName($Today)) {
				$Cell['_attr']['style'] = STYLE_DAY_HOLIDAY;
			}
			else {
				if(0 == date("w", $Day)){
					$Cell['_attr']['style'] = STYLE_DAY_HOLIDAY;
				}
				else {
					$Cell['_attr']['style'] = STYLE_DAY_DEFAULT;
				}
			}
		}
		$Index = 0;
		foreach($TargetData as $Src) {
			if($Cell['_attr']['id'] == $Src['_attr']['id']){
				$TargetData[$Index] = $Cell;
				return true;
			}
			++$Index;
		}
		// 見つからない場合は、ここで追加する
		$TargetData['cells'][] = $Cell;
		return true;
	}

	/**
	 * カレンダー情報 ヘッダーセルデータ取得
	 *
	 * @param array $Cells			ヘッダーセルデータ配列
	 */
	function getDataCellsHeader(&$Cells) {
		$Cells = $this->calendardata['cells']['header']['cell'];
	}

	/**
	 * カレンダー情報 曜日セルデータ取得
	 *
	 * @param array $Cells			曜日セルデータ配列
	 */
	function getDataCellsWeek(&$Cells) {
		$Cells = $this->calendardata['cells']['week']['cell'];
	}

	/**
	 * カレンダー情報 日付セルデータ取得
	 *
	 * @param array $Cells			日付セルデータ配列
	 */
	function getDataCellsDay(&$Cells) {
		$Cells = $this->calendardata['cells']['day']['cell'];
	}

	/**
	 * 設定情報のファイル保存
	 *
	 * @return true / false : 書き込み成功 / 書き込み失敗
	 */
	function SaveConfig() {
		if(USE_DEMO) {
			return true;
		}
		else if(!$this->WriteXmlFile($this->configfile, $this->configdata)) {
			$this->errormessage[] = "設定情報 の書き込みに失敗しました";
			return false;
		}
		return true;
	}

	/**
	 * カレンダー情報のファイル保存
	 *
	 * @return true / false : 書き込み成功 / 書き込み失敗
	 */
	function SaveData() {
		if(USE_DEMO) {
			return true;
		}
		$File = $this->datadir . sprintf("/%04d_%02d.xml", $this->cur_year, $this->cur_month);
		if(!$this->WriteXmlFile($File, $this->calendardata)) {
			$this->errormessage[] = "カレンダー情報 の書き込みに失敗しました";
			return false;
		}
		return true;
	}

	/**
	 * HTMLカレンダー表示用データの取得
	 *
	 * @param array $ViewData (out)		表示用データ
	 * @return true / false : 取得成功  / 取得失敗
	 */
	function getHtmlViewData(&$ViewData) {
		$ViewData = array('year' => $this->calendardata['year'],
						  'month' => $this->calendardata['month'],
						  'tableclass' => $this->configdata['tablestyle']['cssclass']['value'],
						  'headercells' => array(),
						  'weekcells' => array(),
						  'daycells' => array());

		$Year = (int)date("Y");
		$Month = (int)date("n");
		$Today = -9999;
		$NotOutPut = -9999;
		$InMonthDays = YSC_CalendarUtil::days_in_month($this->cur_year, $this->cur_month);
		if(($Year == $this->cur_year) && ($Month == $this->cur_month)) {
			if(0 != (int)$this->configdata['common']['todaystyle']) {
				$Today = (int)date("j");
			}
			if(0 == (int)$this->configdata['common']['pastoutput']) {
				$NotOutPut = (int)date("j") - 1;
			}
		}
		else if(($Year > $this->cur_year) ||
			    (($Year == $this->cur_year) && ($Month > $this->cur_month))) {
			if(0 == (int)$this->configdata['common']['pastoutput']) {
			    $NotOutPut = $InMonthDays;
			}
		}
		$Holiday = (int)$this->configdata['common']['makeupholiday'];
		$ReguFlg = $this->MakeRegularHolidayList($ReguList);
		$PastString = $this->configdata['common']['paststring'];
		$RegHolidayString = $this->configdata['common']['regularholidaytext'];

		foreach($this->calendardata['cells'] as $TargetName => $Cells) {
			if('day' == $TargetName)
				$TargetCells = &$ViewData['daycells'];
			else if('week' == $TargetName)
				$TargetCells = &$ViewData['weekcells'];
			else if('header' == $TargetName)
				$TargetCells = &$ViewData['headercells'];
			else
				return false;
			$Week = 0;
			foreach($Cells['cell'] as $Cell){
				$StyleName = $Cell['_attr']['style'];
				if('day' == $TargetName) {
					$RegHoliday = false;
					$Day = (int)$Cell['_attr']['id'];
					if((0 < $Day) && ($InMonthDays >= $Day)) {
						if((0 != $Holiday) && ($Week > 0) && ($Week < 7)) {
							if(YSC_CalendarUtil::ktHolidayName(mktime(0,0,0, $this->cur_month, $Day, $this->cur_year)) != ''){
								$StyleName = STYLE_DAY_HOLIDAY;
							}
						}
						if($ReguFlg) {
							// 定休日表示スタイル
							if($this->IsRegularHoliday($Week, $Day, $ReguList)) {
								$StyleName = STYLE_DAY_REGHOLIDAY;
								$RegHoliday = true;
							}
						}
						// 本日表示スタイルを適用する
						if($Today == $Day) {
							$StyleName = STYLE_DAY_TODAY;
						}
					}
				}
				if(!$this->SearchConfigStyleCellByName($StyleName, $Style)) {
					$this->errormessage[] = "設定情報 セル表示スタイルデータが見つかりません";
					return false;
				}
				$ViewCell = array('id' => $Cell['_attr']['id'],
					  			  'tdclass' => $Style['cssclass']['value'],
					  			  'text' => array());
				foreach($Cell['text'] as $Text){
					if(isset($Text['value'])) {
						$Value = $Text['value'];
					}
					else {
						$Value = '';
					}
					if('day' == $TargetName) {
						if(1 != (int)$Text['_attr']['id']) {
							if((0 < $Day) && ($InMonthDays >= $Day)) {
								if($NotOutPut >= $Day ) {
									// 経過日の状態表示文字列を入れ替える
									$Value = $PastString;
								}
								else if(($RegHoliday) && ('' != $RegHolidayString)) {
									// 定休日の状態表示文字列を入れ替える
									$Value = $RegHolidayString;
								}
							}
						}
					}
					$FontCss = '';
					foreach($Style['text'] as $Stext){
						if($Stext['_attr']['id'] == $Text['_attr']['id']) {
							$FontCss = $Stext['cssclass']['value'];
							break;
						}
					}
					$ViewCell['text'][] = array('id' => $Text['_attr']['id'],
												'fontclass' => $FontCss,
												'value' => $Value);
				}
				$TargetCells[] = $ViewCell;
				unset($ViewCell);
				if(++$Week > 6) $Week = 0;
			}
		}
		return true;
	}

	/**
	 * 設定情報一括 正当性チェック
	 *
	 * @param array $Conf		： 設定情報
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildConfig(&$Conf) {
		$CheckList = array(array('reg' => array('common' => REG_NULL),
								 'msg' => '設定情報のフォーマットが不正です (common)'),
						   array('reg' => array('tablestyle' => REG_NULL),
						   		 'msg' => '設定情報のフォーマットが不正です (tablestyle)'),
						   array('reg' => array('cellstyle' => REG_NULL),
						   		 'msg' => '設定情報のフォーマットが不正です (cellstyle)'));
		if(!$this->IsValidLocal($CheckList, $Conf)){
			return false;
		}

		foreach($Conf as $Key => $Value){
			if(!is_array($Value)) {
				$this->errormessage[] = '共通データのフォーマットが不正です';
				return false;
			}
			switch ($Key) {
				case 'config' :
					if(!$this->IsValidConfigCommon($Value))
						return false;
					break;
				case 'tablestyle' :
					if(!$this->IsVaildConfigTableStyle($Value))
						return false;
					break;
				case 'cellstyle' :
					if(!$this->IsVaildConfigCellStyle($Value))
						return false;
					break;
				default :
					return false;
			}
		}
		return true;
	}

	/**
	 * 設定情報 共通データ 正当性チェック
	 *
	 * @param array $Common		： 共通データ
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildConfigCommon(&$Common) {
		$CheckList = array(array('reg' => array('imgoutput' => REG_ONOFF),
								 'msg' => '画像保存指定が不正です'),
						   array('reg' => array('imgoutputdir' => REG_ANY),
						   		 'msg' => '画像保存先ディレクトリに使用できない文字が含まれています'),
						   array('reg' => array('pastoutput' => REG_ONOFF),
						   		 'msg' => '経過日予約状況表示指定が不正です'),
						   array('reg' => array('paststring' => REG_ANY),
						   		 'msg' => '経過日状況代替文字に登録できない文字が含まれています'),
						   array('reg' => array('makeupholiday' => REG_ONOFF),
						   		 'msg' => '祝日表示指定が不正です'),
						   array('reg' => array('todaystyle' => REG_ONOFF),
						   		 'msg' => '本日表示指定が不正です'),
						   array('reg' => array('headerformat' => REG_MUST),
						   		 'msg' => 'ヘッダー文字列が空か、登録できない文字が含まれています'),
						   array('reg' => array('weektextlist' => REG_WEEK),
						   		 'msg' => '曜日文字列が空か、登録できない文字が含まれています'),
						   array('reg' => array('daytextlist' => REG_MUST),
						   		 'msg' => '状況選択リスト文字列が空か、登録できない文字が含まれています'),
						   array('reg' => array('daytextdefault' => REG_MUST),
						   		 'msg' => '状況初期表示文字列に登録できない文字が含まれています'),
						   array('reg' => array('regularholiday' => REG_ANY),
						   		 'msg' => '定休日指定が不正です'),
						   array('reg' => array('regularholidaytext' => REG_ANY),
						   		 'msg' => '定休日代替文字列に登録できない文字が含まれています'),
						   array('reg' => array('pre_month' => REG_NUM1),
						   		 'msg' => '表示期間１が不正です'),
						   array('reg' => array('fwd_month' => REG_NUM1),
						   		 'msg' => '表示期間２が不正です'),);
		return $this->IsValidLocal($CheckList, $Common);
	}

	/**
	 * 設定情報 テーブル表示スタイルデータ 正当性チェック
	 *
	 * @param array $Style		： スタイルデータ
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildConfigTableStyle(&$Style) {
		$CheckList1 = array(array('reg' => array('headertext' => REG_NUM1),
								  'msg' => 'ヘッダー行数が不正です'),
							array('reg' => array('weektext' => REG_NUM1),
								  'msg' => '曜日行数が不正です'),
							array('reg' => array('daytext' => REG_NUM1),
								  'msg' => '日付行数が不正です'));
		$CheckList2 = array(array('reg' => array('border' => REG_NUM1),
								  'msg' => '罫線太さが不正です'),
							array('reg' => array('bordercolor' => REG_HEXCOLOR),
								  'msg' => '罫線色が不正です'),
							array('reg' => array('daywidth' => REG_NUM1),
								  'msg' => '日付セル幅が不正です'),
   							array('reg' => array('headerheight' => REG_NUM1),
   								  'msg' => 'ヘッダーセル高さが不正です'),
   							array('reg' => array('weekheight' => REG_NUM1),
   								  'msg' => '曜日セル高さが不正です'),
   							array('reg' => array('dayheight' => REG_NUM1),
   								  'msg' => '日付セル高さが不正です'),
							array('reg' => array('cssclass' => REG_ANY),
								  'msg' => 'TDタグ CSSクラス名に登録できない文字が含まれています'));
		$Ret = true;
		$Ret &= $this->IsValidLocal($CheckList1, $Style);
		$Ret &= $this->IsValidLocal($CheckList2, $Style, 'value');
		return $Ret;
	}

	/**
	 * 設定情報 セル表示スタイルデータ 正当性チェック
	 *
	 * @param array $Style		： スタイルデータ
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildConfigCellStyle(&$Style) {
		$CheckList = array(array('reg' => array('cell' => REG_NULL),
								 'msg' => 'セルデータのフォーマットが不正です (cell)'));
		if(!$this->IsValidLocal($CheckList, $Style)){
			return false;
		}
		foreach($Style['cell'] as $Cell){
			if(!$this->IsValidConfigCell($Cell))
				return false;
		}
		return true;
	}

	/**
	 * 設定情報 セルデータ 正当性チェック
	 *
	 * @param array $Cell		： セルデータ
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildConfigCell(&$Cell) {
		$CheckList1	   = array(array('reg' => array('title' => REG_MUST),
									 'msg' => 'セル タイトルが空か、登録できない文字が含まれています'),
							   array('reg' => array('text' => REG_NULL),
							   		 'msg' => 'セルデータのフォーマットが不正です (text)'),
							   array('reg' => array('_attr' => REG_NULL),
							   		 'msg' => 'セルデータのフォーマットが不正です (attr)'));
		$CheckList2	   = array(array('reg' => array('bgcolor' => REG_HEXCOLOR),
									 'msg' => '背景色が不正です'),
							   array('reg' => array('cssclass' => REG_ANY),
							   		 'msg' => 'TDタグ CSSクラス名に登録できない文字が含まれています'));
		$CheckListAttr = array(array('reg' => array('name' => REG_MUST),
									 'msg' => 'セルデータのフォーマットが不正です (name)'),
							   array('reg' => array('target' => REG_MUST),
							   		 'msg' => 'セルデータのフォーマットが不正です (target)'),
							   array('reg' => array('custom' => REG_ONOFF),
							   		 'msg' => 'セルデータのフォーマットが不正です (custom)'));
		$Ret = true;
		$Ret &= $this->IsValidLocal($CheckList1, $Cell);
		$Ret &= $this->IsValidLocal($CheckList2, $Cell, 'value');
		$Ret &= $this->IsValidLocal($CheckListAttr, $Cell['_attr']);
		if(!$Ret) {
			return false;
		}
		if(!is_array($Cell['text'])) {
			$this->errormessage[] = 'セルデータのフォーマットが不正です (text)';
			return false;
		}
		else {
			$Ret = true;
			foreach($Cell['text'] as $Text) {
				$Ret &= $this->IsVaildConfigCellText($Text);
			}
		}
		return $Ret;
	}

	/**
	 * 設定情報 セルテキストデータ 正当性チェック
	 *
	 * @param array $Texts		： テキストデータ
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildConfigCellText(&$Text) {
		$CheckList = array(array('reg' => array('fontsize' => REG_NUM1),
								 'msg' => '文字サイズが不正です'),
						   array('reg' => array('fontcolor' => REG_HEXCOLOR),
								 'msg' => '文字色が不正です'),
						   array('reg' => array('fontfile' => REG_FILE),
								 'msg' => 'フォントファイルに使用できない文字が含まれています'),
						   array('reg' => array('vpadding' => REG_NUM2),
								 'msg' => '文字左右表示位置が不正です'),
						   array('reg' => array('hpadding' => REG_NUM2),
								 'msg' => '文字上下表示位置が不正です'),
						   array('reg' => array('cssclass' => REG_ANY),
								 'msg' => '文字列 CSSクラス名に登録できない文字が含まれています'),
						   array('reg' => array('_attr' => REG_NULL),
								 'msg' => 'セルデータのフォーマットが不正です (attr)'));

		$CheckListAttr = array(array('reg' => array('id' => REG_NUM1),
								  	 'msg' => 'セルデータのフォーマットが不正です (id)'));
		$Ret = true;
		$Ret &= $this->IsValidLocal($CheckList, $Text, 'value');
		$Ret &= $this->IsValidLocal($CheckListAttr, $Text['_attr']);
		return $Ret;
	}

	/**
	 * カレンダー情報 一括 正当性チェック
	 *
	 * @param array $CalData	：  カレンダー設定
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildData(&$CalData) {
		$CheckList	    = array(array('reg' => array('year' => REG_YEAR),
									  'msg' => '年指定が不正です'),
							    array('reg' => array('month' => REG_MONTH),
									  'msg' => '月指定が不正です'),
							    array('reg' => array('imgfile' => REG_ANY),
									  'msg' => '画像ファイル名が不正です'),
							    array('reg' => array('cells' => REG_NULL),
									  'msg' => 'カレンダー情報のフォーマットが不正です (cells)'));
		$CheckListCells = array(array('reg' => array('header' => REG_NULL),
									  'msg' => 'カレンダー情報のフォーマットが不正です (header)'),
								array('reg' => array('week' => REG_NULL),
									  'msg' => 'カレンダー情報のフォーマットが不正です (week)'),
								array('reg' => array('day' => REG_NULL),
									  'msg' => 'カレンダー情報のフォーマットが不正です (day)'));

		if(!$this->IsValidLocal($CheckList, $CalData)){
			return false;
		}
		if(!$this->IsValidLocal($CheckListCells, $CalData['cells'])){
			return false;
		}
		if(!is_array($CalData['cells'])){
			$this->errormessage[] = 'カレンダー情報のフォーマットが不正です (cells)';
			return false;
		}
		foreach($CalData['cells'] as $Target => $Value){
			if(!is_array($Value['cell'])){
				$this->errormessage[] = 'カレンダー情報のフォーマットが不正です (cell)';
				return false;
			}
			foreach($Value['cell'] as $Cell) {
				if(!$this->IsVaildDataCell($Target, $Cell))
					return false;
			}
		}
		return true;
	}

	/**
	 * カレンダー情報  セル設定  正当性チェック
	 *
	 * @param string $Target	：  対象セル情報種別(header|week|day)
	 * @param array $Cells		：  セル設定
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildDataCell($Target, &$Cell) {
		$CheckList	   = array(array('reg' => array('_attr' => REG_NULL),
									 'msg' => 'セルデータのフォーマットが不正です (cell)'),
							   array('reg' => array('text' => REG_NULL),
									 'msg' => 'セルデータのフォーマットが不正です (text)'));
		$CheckListAttr = array(array('reg' => array('id' => REG_NUMERIC),
									 'msg' => 'セルデータのフォーマットが不正です (id)'),
							   array('reg' => array('style' => REG_MUST),
									 'msg' => 'セルデータのフォーマットが不正です (style)'));
		if(!$this->IsValidLocal($CheckList, $Cell)){
			return false;
		}
		if(!$this->IsValidLocal($CheckListAttr, $Cell['_attr'])){
			return false;
		}
		if(!is_array($Cell['text'])) {
			$this->errormessage[] = 'カレンダー情報のフォーマットが不正です (text)';
			return false;
		}
		foreach($Cell['text'] as $Text){
			if(!$this->IsVaildDataCellText($Target, $Text))
				return false;
		}
		return true;
	}

	/**
	 * カレンダー情報  セルテキストデータ  正当性チェック
	 *
	 * @param string $Target	：  対象セル情報種別(header|week|day)
	 * @param array $Text		：  テキストデータ
	 * @return true / false : 正当 / 不正
	 */
	function IsVaildDataCellText($Target, $Text) {
		$CheckList	   = array(array('reg' => array('_attr' => REG_NULL),
									 'msg' => 'カレンダー情報のフォーマットが不正です (attr)'));
		$CheckListAttr = array(array('reg' => array('id' => REG_NUM1),
									 'msg' => 'カレンダー情報のフォーマットが不正です (id)'));
		switch ($Target) {
			case 'header' :
				if(1 == (int)$Text['_attr']['id']) {
					$CheckList[] = array('reg' => array('value' => REG_MUST),
										 'msg' => 'ヘッダー文字列が空か、登録できない文字が含まれています');
				}
				break;
			case 'week' :
				if(1 == (int)$Text['_attr']['id']) {
					$CheckList[] = array('reg' => array('value' => REG_MUST),
										 'msg' => '曜日文字列が空か、登録できない文字が含まれています');
				}
				break;
			case 'day' :
				if(1 == (int)$Text['_attr']['id']) {
					if(isset($Text['value'])) {
						$CheckList[] = array('reg' => array('value' => REG_DATE),
											 'msg' => '日付文字列に登録できない文字が含まれています');
					}
				}
				else {
					if(isset($Text['value'])) {
						$CheckList[] = array('reg' => array('value' => REG_ANY),
											 'msg' => '状況表示文字に登録できない文字が含まれています');
					}
				}
				break;
			default :
				$this->errormessage[] = 'カレンダー情報のフォーマットが不正です (???)';
				return false;
		}

		if(!$this->IsValidLocal($CheckList, $Text)){
			return false;
		}
		if(!$this->IsValidLocal($CheckListAttr, $Text['_attr'])){
			return false;
		}
		return true;
	}

	/**
	 * カレンダー情報ファイルの存在チェック
	 *
	 * @param int $Year				カレンダー年
	 * @param int $Month			カレンダー月
	 * @return true / false : ファイル在り / ファイル無し
	 */
	function IsExistsDataFile($Year, $Month){
		$File = $this->datadir . sprintf("/%04d_%02d.xml", $Year, $Month);
		return file_exists($File);
	}

	/**
	 * XMLデータをファイルから読み込み 配列に展開する
	 *
	 * @param string $File	XMLファイル名
	 * @param array &$Vals	配列に展開したデータ
	 * @return true / false : 成功 / 失敗
	 */
	function LoadXmlFile($File, &$Vals){
		if(!file_exists($File)){
			$this->errormessage[] = "XMLファイルが見つかりません";
			return false;
		}
		$XmlString = file_get_contents($File);
		if (!$XmlString) {
			$this->errormessage[] = "XMLファイルの読み込みができませんでした";
			return false;
		}

		$ForcEnumOpt = array('cell', 'text');

		$ObjXML = new XML_Unserializer();
		$ObjXML->setOption('encoding', 'UTF-8');
		$ObjXML->setOption('contentName', 'value');
		$ObjXML->setOption('parseAttributes',true);
		$ObjXML->setOption('attributesArray', '_attr');
		$ObjXML->setOption('forceEnum', $ForcEnumOpt);
		$ObjXML->unserialize($XmlString);
		$Vals = $ObjXML->getUnserializedData();
		if(!isset($Vals)) {
			$this->errormessage[] = "XMLファイルのパースができませんでした";
			return false;
		}
		return true;
	}
	/**
	 * 配列からXMLに変換しファイルに書き込む
	 *
	 * @param string $File
	 * @param array $Vals
	 * @return true / false : 成功 / 失敗
	 */
	function WriteXmlFile($File, &$Vals){
		$ObjXML = new XML_Serializer();
		$ObjXML->setOption('rootName', 'root');
		$ObjXML->setOption('mode', 'simplexml');
		$ObjXML->setOption('indent', '  ');
		$ObjXML->setOption('encoding', 'UTF-8');
		$ObjXML->setOption('contentName', 'value');
		$ObjXML->setOption('parseAttributes',true);
		$ObjXML->setOption('attributesArray', '_attr');
		$ObjXML->serialize($Vals);
		$XmlString = $ObjXML->getSerializedData();
		if('' == $XmlString || is_object($XmlString)){
			return false;
		}
		if(!YSC_CalendarUtil::filelock($File)) {
			return false;
		}
		$Ret = YSC_CalendarUtil::file_put_contents($File, $XmlString);
		YSC_CalendarUtil::fileunlock($File);
		if(!$Ret) return false;

		return true;
	}

	/**
	 * カレンダーデータの読み込みを行う
	 *
	 * @param int $Year		: 年
	 * @param int $Month	： 月
	 */
	function LoadCalendarData($Year, $Month){
		$File = $this->datadir . sprintf("/%04d_%02d.xml", $Year, $Month);
		if(!$this->LoadXmlFile($File, $this->calendardata)){
			$this->CreateCalendarData($Year, $Month, $this->calendardata);
		}
	}

	/**
	 * カレンダーデータの新規作成
	 *
	 * @param int $Year		: 年
	 * @param int $Month	： 月
	 * @param array(out) $Vals ： カレンダーデータ
	 */
	function CreateCalendarData($Year, $Month, &$Vals) {
		$HeaderText = str_replace(array('{year}', '{month}'),
								  array((string)$Year,(string)$Month),
								  $this->configdata['common']['headerformat']);
		$WekkTexts = explode(',', trim($this->configdata['common']['weektextlist']));

		$Vals = array();
		$Vals['year'] = $Year;
		$Vals['month'] = $Month;
		$Vals['imgfile'] = '';
		$Vals['cells'] = array();
		$Vals['cells']['header'] = array();
		$Vals['cells']['header']['cell'] = array();
		$Vals['cells']['header']['cell'][] =
			array('_attr' => array('id' => '1', 'style' => STYLE_HEADER_DEFAULT),
				  'text' => array(array('value' => $HeaderText, '_attr' => array('id' => '1'))));

		$Vals['cells']['week'] = array();
		$Dst = &$Vals['cells']['week']['cell'];
		$Dst = array();
		for($i = 0; $i < 7; ++$i){
			if($i == 0)			$Style = STYLE_WEEK_HOLIDAY;
			else if($i == 6)	$Style = STYLE_WEEK_WEEKEND;
			else				$Style = STYLE_WEEK_DEFAULT;
			$Dst[$i] = array('_attr' => array('id' => (string)($i + 1), 'style' => $Style),
							 'text' => array(array('value' => $WekkTexts[$i], '_attr' => array('id' => '1'))));
		}
		$Vals['cells']['day'] = array();
		$Dst = &$Vals['cells']['day']['cell'];
		$Dst = array();
		$InMonthDays = YSC_CalendarUtil::days_in_month($Year, $Month);
		$FirstWeek = (int)date("w", mktime(0,0,0, $Month, 1, $Year));
		$CellCnt = 0;
		$DayTexts = array('', '');
		while(true){
			for($Week = 0; $Week < 7; ++$Week, ++$CellCnt) {
				$Day = $CellCnt - $FirstWeek + 1;
				$DayTexts[0] = '';
				$DayTexts[1] = '';
				$Style = STYLE_DAY_DEFAULT;
				if((0 < $Day) && ($InMonthDays >= $Day)){
					if($Week == 0)			$Style = STYLE_DAY_HOLIDAY;
					else if($Week == 6)		$Style = STYLE_DAY_WEEKEND;
					$DayTexts[0] = (string)$Day;
					$DayTexts[1] = $this->configdata['common']['daytextdefault'];
				}
				$Dst[$CellCnt] = array('_attr' => array('id' => (string)$Day, 'style' => $Style), 'text' =>  array());
				for($Tcount = 0; $Tcount < (int)$this->configdata['tablestyle']['daytext']; $Tcount++) {
					$Dst[$CellCnt]['text'][$Tcount] =
						array('value' => ($Tcount == 0 ? $DayTexts[0]: $DayTexts[1]),
							  '_attr' => array('id' => (string)($Tcount + 1)));
				}
			}
			if($Day >= $InMonthDays){
				break;
			}
		}
	}

	/**
	 * 設定情報 定休日文字列をインデックスに変換する
	 *
	 * @param array $List (out)		定休日のインデックスデータ
	 * 				$List['week'] :	0,1,2,3,4,5,6 / 日,月,火,水,木,金,土
	 * 				$List['day']  :	1-31 / 日付
	 * @return true /false ： 定休日設定 / 無し
	 */
	function MakeRegularHolidayList(&$List) {
		static $WeekString = array('日', '月', '火', '水', '木', '金', '土');
		$List = array('week' => array(), 'day' => array());
		if(!isset($this->configdata['common']['regularholiday'])){
			return false;
		}
		$Holidays = explode(',', trim($this->configdata['common']['regularholiday']));
		foreach($Holidays as $Holiday) {
			if(is_numeric($Holiday)){
				$List['day'][] = (int)$Holiday;
			}
			else {
				for($Week = 0; $Week < 7; ++$Week) {
					if($WeekString[$Week] == $Holiday) {
						$List['week'][] = $Week;
						break;
					}
				}
			}
		}
		return true;
	}

	/**
	 * 定休日の判定
	 *
	 * @param int $Week				判定する週のインデックス
	 * @param int $Day				判定する日にち
	 * @param array $List			定休日データ
	 * 								MakeRegularHolidayListで作成したデータ
	 * @return true / false : 定休日 / 定休日ではない
	 */
	function IsRegularHoliday($Week, $Day, &$List) {
		foreach($List['week'] as $Value) {
			if($Value == $Week) {
				return true;
			}
		}
		foreach($List['day'] as $Value) {
			if($Value == $Day) {
				return true;
			}
		}
		return false;
	}

	/**
	 * 設定情報 セルデータの検索
	 *
	 * @param string $Target		検索するセルデータのターゲット属性 (header|week|day)
	 * @param array $Cells			検索条件にマッチしたセルデータ配列
	 * @return true / false : セルデータ有り / セルデータ無し
	 */
	function SearchConfigStyleCell($Target, &$Cells) {
		$Cells = array();
		$Ret = false;
		foreach($this->configdata['cellstyle']['cell'] as $Cell) {
			if($Cell['_attr']['target'] == $Target){
				$Cells[] = $Cell;
				$Ret = true;
			}
		}
		return $Ret;
	}

	/**
	 * 設定情報 セルデータの検索
	 *
	 * @param string $Namet		検索するセルデータの名称属性
	 * @param array $Cells			検索条件にマッチしたセルデータ
	 * @return true / false : セルデータ有り / セルデータ無し
	 */
	function SearchConfigStyleCellByName($Name, &$Cell) {
		foreach($this->configdata['cellstyle']['cell'] as $Src) {
			if($Src['_attr']['name'] == $Name){
				$Cell = $Src;
				return true;
			}
		}
		return false;
	}

	/**
	 * 連想配列名称の存在チェック
	 *
	 * @param array $NameLits	連想配列名の配列
	 * @param array $Data		チェックする配列
	 * @return true / false ; 全て存在する / 存在しない名称がある
	 */
	function RequireItemNameCheck(&$NameList, &$Data) {
		foreach($NameList as $Name){
			if(!array_key_exists($Name, $Data)){
				return false;
			}
		}
		return true;
	}

	/**
	 * データ正当性チェック
	 *
	 * @param array $ChkList		チェックリスト配列 array(array(連想配列名称,チェックパターン))
	 * @param array $Data			チェックデータ
	 * @param string $ChildNode		チェックするデータが子ノード場合に子ノード名称を指定
	 * 								 $Data[連想配列名][$ChildNode]
	 * @return true / false : 正当データ / 不当データ
	 */
	function IsValidLocal(&$ChkList, &$Data, $ChildNode = NULL) {
		$Ret = true;
		foreach($ChkList as $ChkItem) {
			foreach($ChkItem['reg'] as $Name => $Reg) {
				if(!array_key_exists($Name, $Data)) {
					$this->errormessage[] = $ChkItem['msg'];
					$Ret = false;
				}
				else if(REG_NULL != $Reg) {
					if(!is_null($ChildNode)) {
						if(!array_key_exists($ChildNode, $Data[$Name])) {
							$this->errormessage[] = $ChkItem['msg'];
							$Ret = false;
						}
						else if(!preg_match($Reg, $Data[$Name][$ChildNode])) {
							$this->errormessage[] = $ChkItem['msg'];
							$Ret = false;
						}
					}
					else {
						if(!preg_match($Reg, $Data[$Name])) {
							$this->errormessage[] = $ChkItem['msg'];
							$Ret = false;
						}
					}
				}
			}
		}
		return $Ret;
	}

	/**
	 * 日付セル表示文字列のリスト作成
	 *
	 * @return unknown
	 */
	function MakeDaySelectList(){
		if(!isset($this->configdata['common']['daytextlist'])){
			return array('');
		}
		return preg_split("/,/", $this->configdata['common']['daytextlist']);
	}

	/**
	 * 設定情報 セル表示スタイルのTEXTデータ初期値を取得する
	 *
	 * @param int_type $Id			TEXTデータID番号
	 * @return セル表示スタイルデータ初期値
	 */
	function ConfigCellStyleDefault($Id) {
		return array('_attr'     => array('id'    => (string)$Id),
			  		 'fontsize'  => array('_attr' => array('use' => 'image'),
										  'value' => NULL),
					 'fontcolor' => array('_attr' => array('use' => 'image'),
										  'value' => NULL),
					 'fontfile'  => array('_attr'  => array('use' => 'image'),
										  'value' => NULL),
					 'vpadding'  => array('_attr'  => array('use' => 'image'),
										  'value' => NULL),
					 'hpadding'  => array('_attr'  => array('use' => 'image'),
										  'value' => NULL),
					 'cssclass'  => array('_attr'  => array('use' => 'html'),
										  'value' => NULL));
		}


}
?>