<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

if(!defined('REG_YEAR')) {
	// 年チェック パターン文字列(1970-2037)
	define('REG_YEAR', '/^(19[7-9][0-9]|20[0-2][0-9]|203[0-7])$/');
}

if(!defined('REG_MONTH')) {
	// 月チェック パターン文字列
	define('REG_MONTH', '/^([1-9]|1[0-2])$/');
}

define("CAL_WEEK_MONDAY","2");
define("CAL_WEEK_TUESDAY","3");
define("CAL_WEEK_WEDNESDAY","4");

class YSC_CalendarUtil {
	function ktHolidayName($MyDate)
	{
		$cstImplementHoliday = mktime(0,0,0,4,12,1973); // 振替休日施行

		$HolidayName = YSC_CalendarUtil::prvHolidayName($MyDate);
		if ($HolidayName == "") {
			if (YSC_CalendarUtil::Weekday($MyDate) == CAL_WEEK_MONDAY) {
				// 月曜以外は振替休日判定不要
				// 5/6(火,水)の判定はprvHolidayNameで処理済
				// 5/6(月)はここで判定する
				if ($MyDate >= $cstImplementHoliday) {
					$YesterDay = mktime(0,0,0,YSC_CalendarUtil::Month($MyDate),
										  (YSC_CalendarUtil::Day($MyDate) - 1),YSC_CalendarUtil::Year($MyDate));
					$HolidayName = YSC_CalendarUtil::prvHolidayName($YesterDay);
					if ($HolidayName != "") {
						$Result = "振替休日";
					} else {
						$Result = "";
					}
				} else {
				  $Result = "";
				}
			} else {
				$Result = "";
			}
		} else {
			$Result = $HolidayName;
		}

		return $Result;
	}


	function prvHolidayName($MyDate)
	{

		$cstImplementTheLawOfHoliday = mktime(0,0,0,7,20,1948);		// 祝日法施行
		$cstShowaTaiso = mktime(0,0,0,2,24,1989); 				 	// 昭和天皇大喪の礼
		$cstAkihitoKekkon = mktime(0,0,0,4,10,1959);				// 明仁親王の結婚の儀
		$cstNorihitoKekkon = mktime(0,0,0,6,9,1993);				// 徳仁親王の結婚の儀
		$cstSokuireiseiden = mktime(0,0,0,11,12,1990);				// 即位礼正殿の儀

		$MyYear = YSC_CalendarUtil::Year($MyDate);
		$MyMonth = YSC_CalendarUtil::Month($MyDate);
		$MyDay = YSC_CalendarUtil::Day($MyDate);

		if ($MyDate < $cstImplementTheLawOfHoliday)
			return "";	// 祝日法施行以前
		else;

		$Result = "";
		switch ($MyMonth) {
		// １月 //
		case 1:
			if ($MyDay == 1) {
				$Result = "元日";
			} else {
				if ($MyYear >= 2000) {
					$strNumberOfWeek =
							(floor(($MyDay - 1) / 7) + 1) . YSC_CalendarUtil::Weekday($MyDate);
					if ($strNumberOfWeek == "22") {	 //CAL_WEEK_MONDAY:2
						$Result = "成人の日";
					} else;
				} else {
					if ($MyDay == 15) {
						$Result = "成人の日";
					} else;
				}
			}
			break;
		// ２月 //
		case 2:
			if ($MyDay == 11) {
				if ($MyYear >= 1967) {
					$Result = "建国記念の日";
				} else;
			} elseif ($MyDate == $cstShowaTaiso) {
				$Result = "昭和天皇の大喪の礼";
			} else;
			break;
		// ３月 //
		case 3:
			if ($MyDay == YSC_CalendarUtil::prvDayOfSpringEquinox($MyYear)) {	 // 1948～2150以外は[99]
				$Result = "春分の日"; 						 // が返るので､必ず≠になる
			} else;
			break;
		// ４月 //
		case 4:
			if ($MyDay == 29) {
				if ($MyYear >= 2007) {
					$Result = "昭和の日";
				} elseif ($MyYear >= 1989) {
					$Result = "みどりの日";
				} else {
					$Result = "天皇誕生日";
				}
			} elseif ($MyDate == $cstAkihitoKekkon) {
				$Result = "皇太子明仁親王の結婚の儀";
			} else;
			break;
		// ５月 //
		case 5:
			if ($MyDay == 3) {
				$Result = "憲法記念日";
			} elseif ($MyDay == 4) {
				if ($MyYear >= 2007) {
					$Result = "みどりの日";
				} elseif ($MyYear >= 1986) {
					if (YSC_CalendarUtil::Weekday($MyDate) > CAL_WEEK_MONDAY) {
						// 5/4が日曜日は『只の日曜』､月曜日は『憲法記念日の振替休日』(～2006年)
						$Result = "国民の休日";
					} else;
				} else;
			} elseif ($MyDay == 5) {
				$Result = "こどもの日";
			} elseif ($MyDay == 6) {
				if ($MyYear >= 2007) {
					// 07/5/26 if 条件の一番外側のカッコ不足を修正
					if ((YSC_CalendarUtil::Weekday($MyDate) == CAL_WEEK_TUESDAY) || (YSC_CalendarUtil::Weekday($MyDate) == CAL_WEEK_WEDNESDAY)) {
						$Result = "振替休日";		// [5/3,5/4が日曜]ケースのみ、ここで判定
					} else;
				} else;
			} else;
			break;
		// ６月 //
		case 6:
			if ($MyDate == $cstNorihitoKekkon) {
				$Result = "皇太子徳仁親王の結婚の儀";
			} else;
			break;
		// ７月 //
		case 7:
			if ($MyYear >= 2003) {
				$strNumberOfWeek =
						(floor(($MyDay - 1) / 7) + 1) . YSC_CalendarUtil::Weekday($MyDate);
				if ($strNumberOfWeek == "32") {	 //CAL_WEEK_MONDAY:2
					$Result = "海の日";
				} else;
			} elseif ($MyYear >= 1996) {
				if ($MyDay == 20) {
					$Result = "海の日";
				} else;
			} else;
			break;
		// ９月 //
		case 9:
			//第３月曜日(15～21)と秋分日(22～24)が重なる事はない
			$MyAutumnEquinox = YSC_CalendarUtil::prvDayOfAutumnEquinox($MyYear);
			if ($MyDay == $MyAutumnEquinox) {		// 1948～2150以外は[99]
				$Result = "秋分の日"; 			 // が返るので､必ず≠になる
			} else {
				if ($MyYear >= 2003) {
					$strNumberOfWeek =
							(floor(($MyDay - 1) / 7) + 1) . YSC_CalendarUtil::Weekday($MyDate);
					if ($strNumberOfWeek == "32") {	 //CAL_WEEK_MONDAY:2
						$Result = "敬老の日";
					} elseif (YSC_CalendarUtil::Weekday($MyDate) == CAL_WEEK_TUESDAY) {
						if ($MyDay == ($MyAutumnEquinox - 1)) {
							$Result = "国民の休日";
						} else;
					} else;
				} elseif ($MyYear >= 1966) {
					if ($MyDay == 15) {
						$Result = "敬老の日";
					} else;
				} else;
			}
			break;
		// １０月 //
		case 10:
			if ($MyYear >= 2000) {
				$strNumberOfWeek =
						(floor(( $MyDay - 1) / 7) + 1) . YSC_CalendarUtil::Weekday($MyDate);
				if ($strNumberOfWeek == "22") {	 // CAL_WEEK_MONDAY:2
					$Result = "体育の日";
				} else;
			} elseif ($MyYear >= 1966) {
				if ($MyDay == 10) {
					$Result = "体育の日";
				} else;
			} else;
			break;
		// １１月 //
		case 11:
			if ($MyDay == 3) {
				$Result = "文化の日";
			} elseif ($MyDay == 23) {
				$Result = "勤労感謝の日";
			} elseif ($MyDate == $cstSokuireiseiden) {	// 07/04/11 $抜け修正
				$Result = "即位礼正殿の儀";
			} else;
			break;
		// １２月 //
		case 12:
			if ($MyDay == 23) {
				if ($MyYear >= 1989) {
					$Result = "天皇誕生日";
				} else;
			} else;
			break;
		}
		return $Result;
	}

	function prvDayOfSpringEquinox($MyYear)
	{
		if ($MyYear <= 1947)
			$Result = 99; //祝日法施行前
		elseif ($MyYear <= 1979)
			// floor 関数は[VBAのInt関数]に相当
			$Result = floor(20.8357 +
				(0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
		elseif ($MyYear <= 2099)
			$Result = floor(20.8431 +
				(0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
		elseif ($MyYear <= 2150)
			$Result = floor(21.851 +
				(0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
		else
			$Result = 99; //2151年以降は略算式が無いので不明

		return $Result;
	}

	function prvDayOfAutumnEquinox($MyYear)
	{
		if ($MyYear <= 1947)
			$Result = 99; //祝日法施行前
		elseif ($MyYear <= 1979)
			// floor 関数は[VBAのInt関数]に相当
			$Result = floor(23.2588 +
				(0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
		elseif ($MyYear <= 2099)
			$Result = floor(23.2488 +
				(0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
		elseif ($MyYear <= 2150)
			$Result = floor(24.2488 +
				(0.242194 * ($MyYear - 1980)) - floor(($MyYear - 1980) / 4));
		else
			$Result = 99; //2151年以降は略算式が無いので不明

		return $Result;
	}

	function Weekday($MyDate){
		return strftime("%w",$MyDate) + 1;  // 日(1),月(2)‥‥土(7)
	}

	function Year($MyDate){
		return strftime("%Y",$MyDate) - 0;  // 数値で返す
	}

	function Month($MyDate){
		return strftime("%m",$MyDate) - 0;  // 数値で返す
	}

	function Day($MyDate){
		return strftime("%d",$MyDate) - 0;  // 数値で返す
	}

	/**
	 * ファイルロック処理
	 *
	 * @param string $File			： ファイル名
	 * @return true / false : ロック完了 / ロック未完了
	 */
	function filelock($File){
		$Lockfile= $File . ".lock";
		$Retry = 3;

		// 30秒以上古いロックは削除する
		if (file_exists($Lockfile)) {
			$mtime = filemtime($Lockfile);
			if ($mtime < time() - 30) {
				fileunlock($File);
			}
		}

		// ロック用ディレクトリを作成する
		while (!mkdir($Lockfile, 0755)) {
			if ($Retry-- <= 0)
				return false;
			sleep(1);
		}
		return true;
	}

	/**
	 * ファイルアンロック処理
	 *
	 * @param string $File			： ファイル名
	 */
	function fileunlock($File) {
		$Lockfile= $File . ".lock";
		// ロック用ディレクトリを削除する
		if (file_exists($Lockfile)) {
			rmdir($Lockfile);
		}
	}

	/**
	 * ファイルロック状態取得
	 *
	 * @param string $File			： ファイル名
	 * @return true / false : ロックされている / ロックされていない了
	 */
	function filelockstatus($File){
		$Lockfile= $File . ".lock";
		$Retry = 3;

		// ロック用ディレクトリが存在する
		while (file_exists($Lockfile)) {
			if ($Retry-- <= 0)
				return true;
			sleep(1);
		}
		return false;
	}

	/**
	 * ファイル保存
	 * @param $File				ファイル名
	 * @param $String			テキストデータ
	 * @return true / false : 保存成功 / 保存失敗
	 */
	function file_put_contents($File, $String) {
		$hdl = @fopen($File, "w+");
		if(!$hdl) {
			return false;
		}
		if(!fwrite($hdl, $String)) {
			return false;
		}
		fclose($hdl);
		return true;
	}

	/**
	 * 月を加えた年月を取得する
	 *
	 * @param int $AddMonth				月数
	 * @param int $Year	(out)			取得 年
	 * @param int $Month (out			取得 月
	 * @param int $BYear				基準 年
	 * @param int $BMonth				基準月
	 * @return true / false : 取得成功 : パラメータ不正
	 */
	function monthadd($AddMonth, &$Year, &$Month, $BYear = NULL, $BMonth = NULL) {
		$Year = (int)date("Y");
		$Month = (int)date("n");
		if(!is_null($BYear)) {
			if(preg_match(REG_YEAR, (string)$BYear)) {
				$Year = $BYear;
			}
			else
				return false;
		}
		if(!is_null($BMonth)) {
			if(preg_match(REG_MONTH, (string)$BMonth)) {
				$Month = $BMonth;
			}
			else
				return false;
		}
		if($AddMonth > 0) {
			$Year = $Year + floor($AddMonth / 12);
		}
		else {
		    $Year = $Year + ceil($AddMonth / 12);
		}
		$Month = $Month + ($AddMonth % 12);
		if(12 < $Month) {
			++$Year;
			$Month -= 12;
		}
		elseif(1 > $Month){
			--$Year;
			$Month += 12;
		}
		return true;
	}

	/**
	 * 月の日数を取得する
	 * @param $Year				年
	 * @param $Month			月
	 * @return 日数
	 */
	function days_in_month($Year, $Month) {
		$Days = array(31,28,31,30,31,30,31,31,30,31,30,31);
		if(2 == $Month) {
			if((($Year % 4 == 0) && ($Year % 100 != 0)) ||
			   (($Year % 4 == 0) && ($Year % 400 == 0))){
			   	return 29;
			}
		}
		return $Days[$Month - 1];
	}

}
?>