<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

require_once(dirname(__FILE__) . '/ysc_imgcalendarmodel.php');
require_once(dirname(__FILE__) . '/ysc_imgcalendarview.php');
require_once(dirname(__FILE__) . '/ysc_calendarutil.php');

/**
 * Smarty 設定定義
 *
 */
if(!defined('SMARTY_SETTING')) {
define('SMARTY_SETTING', 1);
define('SMARTY_CLASS_FILE', dirname(__FILE__) . '/../smarty/Smarty.class.php');
define('SMARTY_LEFT_DELIMITER', '<!--{');
define('SMARTY_RIGHT_DELIMITER', '}-->');
define('SMARTY_TEMPLATE_DIR', dirname(__FILE__) . '/../templates');
define('SMARTY_CACHE_DIR', dirname(__FILE__) . '/../cashe');
define('SMARTY_CONFIG_DIR', dirname(__FILE__) . '/../configs');
define('SMARTY_COMPILE_DIR', dirname(__FILE__) . '/../templates_c');
}
require_once(SMARTY_CLASS_FILE);


/**
 * 画像タイプ定義
 *
 */
if(!defined('CAL_IMG_TYPE')) {
define('CAL_IMG_TYPE', 1);
define('CAL_IMG_TYPE_JPEG', 'jpeg');			// JPGフォーマット
define('CAL_IMG_TYPE_JPG', 'jpg');				// JPGフォーマット
define('CAL_IMG_TYPE_PNG', 'png');				// PNGフォーマット
define('CAL_IMG_TYPE_GIF', 'gif');				// GIFフォーマット
}
/**
 * 出力タイプ定義
 */
if(!defined('CAL_OUTPUT_TYPE')) {
define('CAL_OUTPUT_TYPE', 1);
define('CAL_OUTPUT_IMG', 1);				// 画像出力
define('CAL_OUTPUT_HTML', 0);				// HTML出力
}

/**
 * HTMLカレンダーテンプレートファイル
 */
define('TPL_MOBILE_CALENDAR',	  'mobile_calendar.tpl');
define('TPL_MOBILE_CONTENTSTYPE', 'xhtml');
define('TPL_MOBILE_CHARSET', 	  'shift_jis');
define('TPL_MOBILE_ENCODE', 	  'SJIS');

/**
 * 画像カレンダークラス
 *
 */
class YSC_ImageCalendar {

	/**
	 * 画像カレンダーモデルクラス
	 *
	 * @var YSC_ImageCalendarModel
	 */
	var $model = NULL;

	/**
	 * 画像カレンダービュークラス
	 *
	 * @var YSC_ImageCalendarModelView
	 */
	var $view = NULL;

	/**
	 * 出力タイプ指定
	 *
	 * @var string CAL_OUTPUT_IMG or CAL_OUTPUT_HTML
	 */
	var $output_type = CAL_OUTPUT_HTML;

	/**
	 * HTML出力時のオプションデータ
	 *
	 * @var  mixed
	 */
	var $viewoption = NULL;

	/**
	 * コンストラクタ
	 *
	 * @param string $ConfFile			設定情報ファイル
	 * @param string_type $DataDir		カレンダー情報格納ディレクトリ
	 * @param int $Year					表示カレンダー年
	 * @param int $Month				表示カレンダー月
	 */
	function YSC_ImageCalendar($ConfFile, $DataDir, $Year = 0, $Month = 0, $OutoutType = CAL_OUTPUT_HTML) {
		$this->model = new YSC_ImageCalendarModel($ConfFile, $DataDir, $Year, $Month);
		if(!$this->model) {
			trigger_error("システムエラーが発生しました", E_USER_ERROR);
		}
		if(CAL_OUTPUT_IMG == $OutoutType) {
			$this->view = new YSC_ImageCalendarView();
		}
		else {
			$this->view = new Smarty();
			$this->view->left_delimiter = SMARTY_LEFT_DELIMITER;
			$this->view->right_delimiter = SMARTY_RIGHT_DELIMITER;
			$this->view->template_dir = SMARTY_TEMPLATE_DIR;
			$this->view->cache_dir = SMARTY_CACHE_DIR;
			$this->view->config_dir = SMARTY_CONFIG_DIR;
			$this->view->compile_dir = SMARTY_COMPILE_DIR;
		}
		$this->output_type = $OutoutType;
	}

	/**
	 * HTML表示オプションデータの設定
	 *
	 * @param array $options		追加オプションデータ配列
	 * 								array(array('Key'=>$Value))
	 */
	function setViewOptions(&$options) {
		$this->viewoption = $options;
	}

	/**
	 * HTML表示オプションデータの全消去
	 *
	 */
	function clearViewOptions() {
		$this->viewoption = array();
	}

	/**
	 * 出力タイプ設定
	 *
	 * @param strig $Type			出力タイプ
	 * 								CAL_OUTPUT_IMG / CAL_OUTPUT_HTML : 画像出力 / HTML出力
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setOutoutType($Type) {
		if(CAL_OUTPUT_IMG != $Type && CAL_OUTPUT_HTML != $Type) {
			return false;
		}
		else {
			if($this->output_type != $Type) {
				unset($this->view);
				if(CAL_OUTPUT_IMG == $OutoutType) {
					$this->view = new YSC_ImageCalendarView();
				}
				elseif(CAL_OUTPUT_HTML == $OutoutType) {
					$this->view = new Smarty();
					$this->view->left_delimiter = SMARTY_LEFT_DELIMITER;
					$this->view->right_delimiter = SMARTY_RIGHT_DELIMITER;
					$this->view->template_dir = SMARTY_TEMPLATE_DIR;
					$this->view->cache_dir = SMARTY_CACHE_DIR;
					$this->view->config_dir = SMARTY_CONFIG_DIR;
					$this->view->compile_dir = SMARTY_COMPILE_DIR;
				}
				else{
					return false;
				}
				if(!$this->view) {
					trigger_error("システムエラーが発生しました", E_USER_ERROR);
				}
				$this->output_type = $OutoutType;
			}
		}
		return false;
	}

	/**
	 * 出力タイプ取得
	 *
	 * @param strig $Type			出力タイプ
	 * 								CAL_OUTPUT_IMG / CAL_OUTPUT_HTML : 画像出力 / HTML出力
	 */
	function getOutoutType($Type) {
		$Type = $this->output_type;
	}

	/**
	 * 設定情報ファイルの設定
	 *
	 * @param stirng $File			ファイル名
	 * @return true / false : 設定成功 / 設定失敗
	 */
	function setConfigFile($File) {
		return $this->model->setConfigFile($File);
	}

	/**
	 * 設定情報ファイル名の取得
	 *
	 * @param string $File (out)	ファイル名
	 */
	function getConfigFile(&$File) {
		$this->model->getConfigFile($File);
	}

	/**
	 * カレンダー情報保存ディレクトリの設定
	 *
	 * @param string $Dir			ディレクトリ名
	 * @return true / false : 設定成功 / 設定失敗
	 * 	 */
	function setDataDir($Dir) {
		return $this->model->setDataDir($Dir);
	}

	/**
	 * カレンダー情報保存ディレクトリの取得
	 *
	 * @param string $Dir (out)		ディレクトリ名
	 */
	function getDataDir(&$Dir) {
		$this->model->setDataDir($Dir);
	}

	/**
	 * カレンダー情報年月の設定
	 *
	 * @param int $Year				年
	 * @param int $Month			月
	 * @return true / false : 設定成功 / 年月指定が不正
	 */
	function setYearMonth($Year, $Month) {
		return $this->model->setYearMonth($Year, $Month);
	}

	/**
	 * カレンダー情報年月の取得
	 *
	 * @param int $Year (out)		年
	 * @param int $Month (out)		月
	 */
	function getYearMonth(&$Year, &$Month) {
		$this->model->getYearMonth($Year, $Month);
	}

	/**
	 * 出力画像タイプの取得
	 *
	 * @param string $Type			画像タイプ
	 * 								CAL_IMG_TYPE_JPEG,CAL_IMG_TYPE_JPG
	 * 								CAL_IMG_TYPE_GIF, CAL_IMG_TYPE_PNG
	 */
	function getImageType(&$Type) {
		$this->view->getImageType($Type);
	}

	/**
	 * 出力画像タイプの設定
	 *
	 * @param string $Type			画像タイプ
	 * 								CAL_IMG_TYPE_JPEG,CAL_IMG_TYPE_JPG
	 * 								CAL_IMG_TYPE_GIF, CAL_IMG_TYPE_PNG
	 * @return true/ false : 設定成功 / 設定失敗
	 */
	function setImageType(&$Type) {
		return $this->view->setImageType($Type);
	}

	/**
	 * アクション関数
	 *
	 * @param int $ActNo			アクションNO
	 * 								Any: 画像カレンダー出力
	 */
	function Action() {
		// 画像出力
		if(CAL_OUTPUT_IMG == $this->output_type) {
			$this->OutputImage();
		}
		// HTML出力
		elseif(CAL_OUTPUT_HTML ==  $this->output_type) {
			$this->OutputHtml();
		}
	}

	/**
	 * 画像出力
	 *
	 */
	function OutputImage() {
		$this->model->getData($CalData);
		if(!$CalData) {
			trigger_error("カレンダー情報が取得できませんでした", E_USER_ERROR);
		}
		$this->model->getConfigCommon($Common);
		if(!$Common) {
			trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
		}
		// 画像ファイル出力「する」設定の場合
		if(1 == (int)$Common['imgoutput']) {
			$Restore = false;
			$ImgSave = false;
			$File = $CalData['imgfile'];
			if('' != $File) {
				if(file_exists($File)) {
					$ImgTime = filemtime($CalData['imgfile']);
					$this->getConfigFile($ConfFile);
					$ConfTime = filemtime($ConfFile);
					$ImgDate = date("z", filemtime($CalData['imgfile']));
					$Today = date("z");
					// 画像ファイルの日付が今日ではない or 設定情報が更新されている場合
					if(($ImgDate <> $Today) || ($ImgTime < $ConfTime)) {
						$ImgSave = true;
					}
				}
				else {
					$Restore = true;
				}
			}
			else {
				$Restore = true;
			}

			if($Restore) {
				// ディレクトリ名にセパレータが付加されている場合は、取り除く
				$Dir = $Common['imgoutputdir'];
				if(substr($Dir, -1) == '/'){
					$Dir = substr($Dir, 0, strlen($Dir) - 1);
				}
				if(file_exists($Dir)) {
					$this->view->getImageType($Ext);
					$File = $Dir . sprintf("/%04d_%02d", $CalData['year'], $CalData['month']) . '.' . $Ext;
					$ImgSave = true;
				}
				else {
					trigger_error("画像カレンダー保存ディレクトリ指定が不正です", E_USER_ERROR);
				}
			}

			if($ImgSave) {
				if(!$this->model->getImageViewData($VData)) {
					trigger_error($this->model->errormessage[0], E_USER_ERROR);
				}
				$VData['imgfile'] = $File;
				if(!$this->view->setViewData($VData)) {
					trigger_error($this->model->errormessage[0], E_USER_ERROR);
				}
				// カレンダー画像保存
				if(!$this->view->Save()) {
					trigger_error($this->view->errormessage, E_USER_ERROR);
				}
			}
			// 画像出力
			if(!$this->view->DisplayFile($File)){
				trigger_error($this->view->errormessage, E_USER_ERROR);
			}
		}
		else {
			if(!$this->model->getImageViewData($VData)) {
				trigger_error($this->model->errormessage[0], E_USER_ERROR);
			}
			if(!$this->view->setViewData($VData)) {
				trigger_error($this->model->errormessage[0], E_USER_ERROR);
			}
			if(!$this->view->Display()) {
				trigger_error($this->view->errormessage[0], E_USER_ERROR);
			}
		}
	}

	/**
	 * HTML出力
	 *
	 */
	function OutputHtml() {
		$this->model->getConfigCommon($Common);
		if(!$Common) {
			trigger_error("設定情報 共通データが取得できませんでした", E_USER_ERROR);
		}
		$this->model->getYearMonth($CurYear, $CurMonth);
		$ListCnt = (int)$Common['pre_month'] + (int)$Common['fwd_month'] + 1;
		$DispLimitString = array();
		$DispLimitOffset = array();
		$Offset = (int)$Common['pre_month'] * -1;
		$Index = 0;
		for($Cnt = 0; $Cnt < $ListCnt; ++$Cnt, ++$Offset){
			YSC_CalendarUtil::monthadd($Offset, $Year, $Month);
			$DispLimitString[$Cnt] = mb_convert_encoding($Year . '年' . $Month . '月', TPL_MOBILE_ENCODE, "UTF-8");
			$DispLimitOffset[$Cnt] = $Offset;
			if(($CurYear == $Year) && ($CurMonth == $Month)) {
				$Index = $Offset;
			}
		}
		$URLImage = mb_convert_encoding($_SERVER['SCRIPT_NAME'] . '?type=' . CAL_OUTPUT_IMG, TPL_MOBILE_ENCODE, "UTF-8");

		// 表示用パラメータ設定
		$this->view->assign('year', $CurYear);
		$this->view->assign('month', $CurMonth);
		$this->view->assign('displimitstring', $DispLimitString);
		$this->view->assign('displimitoffset', $DispLimitOffset);
		$this->view->assign('displimitindex', $Index);
		$this->view->assign('URLImage', $URLImage);
		$this->view->assign('ScriptName', $_SERVER['SCRIPT_NAME']);
		if(isset($this->viewoption)) {
			$this->view->assign('options', $this->viewoption);
		}
		// ヘッダー出力
		if('xhtml' == TPL_MOBILE_CONTENTSTYPE && !isset($this->viewoption['mfd'])) {
			header("Content-Type: application/xhtml+xml; charset=" . TPL_MOBILE_CHARSET);
		}
		// 表示
		$this->view->display(TPL_MOBILE_CALENDAR);
	}
}
?>