<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

require_once(dirname(__FILE__) . '/ysc_calendarmodel.php');


/**
 * 画像カレンダーモデルクラス
 *
 */
class YSC_ImageCalendarModel extends YSC_CalendarModel {
	/**
	 * 画像カレンダー表示用データの取得
	 *
	 * @param array $ViewData (out)		表示用データ
	 * @return true / false : 取得成功  / 取得失敗
	 */
	function getImageViewData(&$ViewData) {
		if(!isset($this->calendardata) || !isset($this->configdata)){
			$this->errormessage = "設定情報またはカレンダー情報が取得されていません";
			return false;
		}
		$ConfTblStyle = &$this->configdata['tablestyle'];
		$ViewData = array('year' => $this->cur_year,
						  'month' => $this->cur_month,
						  'imgfile' => $this->calendardata['imgfile'],
						  'bordercolor' => $this->configdata['tablestyle']['bordercolor']['value'],
						  'cells' => array());
		$TargetCells = &$ViewData['cells'];
		$Year = (int)date("Y");
		$Month = (int)date("n");
		$Today = -9999;
		$NotOutPut = -9999;
		$InMonthDays = YSC_CalendarUtil::days_in_month($this->cur_year, $this->cur_month);
		$Dayrow = 0;
		if(($Year == $this->cur_year) && ($Month == $this->cur_month)) {
			if(0 != (int)$this->configdata['common']['todaystyle']) {
				$Today = (int)date("j");
			}
			if(0 == (int)$this->configdata['common']['pastoutput']) {
				$NotOutPut = (int)date("j") - 1;
			}
		}
		else if(($Year > $this->cur_year) ||
			    (($Year == $this->cur_year) && ($Month > $this->cur_month))) {
			if(0 == (int)$this->configdata['common']['pastoutput']) {
			    $NotOutPut = $InMonthDays;
			}
		}
		$Holiday = (int)$this->configdata['common']['makeupholiday'];
		$ReguFlg = $this->MakeRegularHolidayList($ReguList);
		$PastString = $this->configdata['common']['paststring'];
		$RegHolidayString = $this->configdata['common']['regularholidaytext'];

		foreach($this->calendardata['cells'] as $TargetName => $Cells) {
			if('day' == $TargetName) {
				$X1 = (int)$ConfTblStyle['border']['value'];
				$X2 = (int)$X1 + (int)$ConfTblStyle['daywidth']['value'] - 1;
				$Y1 = ((int)$ConfTblStyle['border']['value'] * 3) +
					  (int)$ConfTblStyle['headerheight']['value'] +
					  (int)$ConfTblStyle['weekheight']['value'];
				$Y2 = $Y1 + (int)$ConfTblStyle['dayheight']['value'] - 1;
			}
			else if('week' == $TargetName) {
				$X1 = (int)$ConfTblStyle['border']['value'];
				$X2 = $X1 + (int)$ConfTblStyle['daywidth']['value'] - 1;
				$Y1 = ((int)$ConfTblStyle['border']['value'] * 2) +
					  (int)$ConfTblStyle['headerheight']['value'];
				$Y2 = $Y1 + (int)$ConfTblStyle['weekheight']['value'] - 1;
			}
			else if('header' == $TargetName) {
				$X1 = (int)$ConfTblStyle['border']['value'];
				$X2 = ($X1 + (int)$ConfTblStyle['daywidth']['value']) * 7 - 1;
				$Y1 = (int)$X1;
				$Y2 = $Y1 + (int)$ConfTblStyle['headerheight']['value'] - 1;
			}
			else
				return false;
			$Week = 0;
			foreach($Cells['cell'] as $Cell){
				$StyleName = $Cell['_attr']['style'];
				if('day' == $TargetName) {
					$RegHoliday = false;
					$Day = (int)$Cell['_attr']['id'];
					if((0 < $Day) && ($InMonthDays >= $Day)) {
						if((0 != $Holiday) && ($Week > 0) && ($Week < 7)) {
							if(YSC_CalendarUtil::ktHolidayName(mktime(0,0,0, $this->cur_month, $Day, $this->cur_year)) != ''){
								$StyleName = STYLE_DAY_HOLIDAY;
							}
						}
						if($ReguFlg) {
							// 定休日表示スタイル
							if($this->IsRegularHoliday($Week, $Day, $ReguList)) {
								$StyleName = STYLE_DAY_REGHOLIDAY;
								$RegHoliday = true;
							}
						}
						// 本日表示スタイルを適用する
						if($Today == $Day) {
							$StyleName = STYLE_DAY_TODAY;
						}
					}
				}
				if(!$this->SearchConfigStyleCellByName($StyleName, $Style)) {
					$this->errormessage = "設定情報 セル表示スタイルデータが見つかりません";
					return false;
				}
				$ViewCell = array('position' => array('x1' => $X1, 'y1' => $Y1, 'x2' => $X2, 'y2' => $Y2),
					  			  'bgcolor' => $Style['bgcolor']['value'],
					  			  'text' => array());

				foreach($Cell['text'] as $Text){
					$Value = $Text['value'];
					if('day' == $TargetName) {
						if(1 != (int)$Text['_attr']['id']) {
							if((0 < $Day) && ($InMonthDays >= $Day)) {
								if($NotOutPut >= $Day ) {
									// 経過日の状態表示文字列を入れ替える
									$Value = $PastString;
								}
								else if(($RegHoliday) && ('' != $RegHolidayString)) {
									// 定休日の状態表示文字列を入れ替える
									$Value = $RegHolidayString;
								}
							}
						}
					}
					foreach($Style['text'] as $Stext){
						if($Text['_attr']['id'] == $Stext['_attr']['id']) {
							$ViewCell['text'][] =
								array('value' => $Value,
									  'fontfile' => $Stext['fontfile']['value'],
									  'fontsize' => (int)$Stext['fontsize']['value'],
									  'fontcolor' => $Stext['fontcolor']['value'],
									  'x1' => $X1 + (int)$Stext['vpadding']['value'],
									  'y1' => $Y1 + (int)$Stext['hpadding']['value']);
							break;
						}
					}
				}
				$TargetCells[] = $ViewCell;
				unset($ViewCell);
				++$Week;
				if(7 == $Week){
					// 次行左端に設定
					$X1 = (int)$ConfTblStyle['border']['value'];
					$X2 = $X1 + (int)$ConfTblStyle['daywidth']['value'] - 1;
					$Y1 = $Y2 + (int)$ConfTblStyle['border']['value'] + 1;
					$Y2 = $Y1 + (int)$ConfTblStyle['dayheight']['value'] - 1;
					$Week = 0;
					if('day' == $TargetName) {
						++$Dayrow;
					}
				}
				else{
					// 右横セル描画位置を設定
					$X1 = $X2 + (int)$ConfTblStyle['border']['value'] + 1;
					$X2 = $X1 + (int)$ConfTblStyle['daywidth']['value'] - 1;
				}
			}
		}
		$ViewData['imgwidth'] = ((int)$ConfTblStyle['daywidth']['value'] * 7) +
								((int)$ConfTblStyle['border']['value'] * 8);
		$ViewData['imgheight'] = (int)$ConfTblStyle['headerheight']['value'] +
								 (int)$ConfTblStyle['weekheight']['value'] +
								 ((int)$ConfTblStyle['dayheight']['value'] * $Dayrow) +
				  				 ((int)$ConfTblStyle['border']['value'] * ($Dayrow + 2)) + 1;
		return true;
	}
}
?>