<?php
/**
 * MRCalender
 *
 * @package		MRCalendar
 * @version		0.60
 * @license 	本プログラムは、フリーソフトウェアです。
 * 				但し、HTML出力画面にあるプログラム名、プログラムバージョンを著作者の承認なし
 * 				に削除しての使用・改変・再配布することは禁止します。
 * @copyright	YS-Lab
 * @author 		大和英喜
 * @link 		http://www.ys-lab.jp
 */

require_once(dirname(__FILE__) . '/ysc_calendarutil.php');

/**
 * 画像タイプ定義
 *
 */
if(!defined('CAL_IMG_TYPE')) {
define('CAL_IMG_TYPE', 1);
define('CAL_IMG_TYPE_JPEG', 'jpeg');			// JPGフォーマット
define('CAL_IMG_TYPE_JPG', 'jpg');				// JPGフォーマット
define('CAL_IMG_TYPE_PNG', 'png');				// PNGフォーマット
define('CAL_IMG_TYPE_GIF', 'gif');				// GIFフォーマット
}

/**
 * データ検証用パターン定義
 *
 */
if(!defined('REG_HEXCOLOR')) {
define('REG_HEXCOLOR', '/^[0-9a-fA-F]{6}$/');	// HEXカラーチェック用パーターン
}

define('DISPLAY_CREDIT', 1);

/**
 * 画像カレンダービュークラス
 *
 */
class YSC_ImageCalendarView {
	var $imgformat = CAL_IMG_TYPE_GIF;
	var $viewdata = NULL;
	var $image = NULL;
	var $gd_info = '';

	var $errormessage = '';

	/**
	 * コンストラクタ
	 *
	 * @param string $ImgType			画像タイプ
	 * @param string_type $ViewData		表示用データ
	 */
	function YSC_ImageCalendarView($ViewData = NULL, $ImgType = '') {
		if('' != $ImgType) {
			if(!$this->setImageType($ImgType)) {
				trigger_error("画像フォーマットの指定が不正です", E_USER_ERROR);
			}
		}

		if(!is_null($ViewData)) {
			if(!$this->setViewData($ViewData)) {
				trigger_error("表示データフォーマットが不正です", E_USER_ERROR);
			}
		}
		$this->gd_info = gd_info();
	}

	/**
	 * 出力画像タイプの取得
	 *
	 * @param string $Type			画像タイプ
	 * 								CAL_IMG_TYPE_JPEG,CAL_IMG_TYPE_JPG
	 * 								CAL_IMG_TYPE_GIF, CAL_IMG_TYPE_PNG
	 */
	function getImageType(&$Type) {
		$Type = $this->imgformat;
	}

	/**
	 * 出力画像タイプの設定
	 *
	 * @param string $Format		画像タイプ
	 * 								CAL_IMG_TYPE_JPEG,CAL_IMG_TYPE_JPG
	 * 								CAL_IMG_TYPE_GIF, CAL_IMG_TYPE_PNG
	 * @return true/ false : 設定成功 / 設定失敗
	 */
	function setImageType(&$Type) {
		if((CAL_IMG_TYPE_GIF == $Type) ||
		   (CAL_IMG_TYPE_JPG == $Type) ||
		   (CAL_IMG_TYPE_JPEG == $Type) ||
		   (CAL_IMG_TYPE_PNG == $Type)) {
			$this->imgformat = $Type;
			return true;
		}
		return false;
	}

	/**
	 * 表示データの取得
	 *
	 * @param array $Data			表示データ
	 */
	function getViewData(&$Data) {
		$Data = $this->viewdata;
	}

	/**
	 * 表示データの設定
	 *
	 * @param array $Data			表示データ
	 */
	function setViewData(&$Data) {
		// TODO : データ正当性チェック追加
		$this->viewdata = $Data;
		return true;
	}

	/**
	 * 画面表出力処理
	 *
	 * @param int $DispNo			出力画面番号
	 * 								ANY: 画像カレンダー出力
	 * @return true /false : 表示出力 / 表示エラー
	 */
	function Display() {
		if(!isset($this->viewdata)) {
			$this->errormessage = "カレンダー表示データが設定されていません";
			return false;
		}
		$this->OutputHeader($this->imgformat);
		if(!$this->OutputImage()) {
			$this->errormessage = "画像データの出力に失敗しました";
			return false;
		}
		return true;
	}

	/**
	 * 画像ファイル出力処理
	 *
	 * @param string $ImageFile		出力する画像ファイル
	 * @return true /false : 正常出力 / 出力エラー
	 */
	function DisplayFile($ImageFile) {
		$ExtStr = pathinfo($ImageFile, PATHINFO_EXTENSION);
		$ExtStr = strtolower($ExtStr);
		if((CAL_IMG_TYPE_GIF == $ExtStr) ||
		   (CAL_IMG_TYPE_JPG == $ExtStr) ||
		   (CAL_IMG_TYPE_JPEG == $ExtStr) ||
		   (CAL_IMG_TYPE_PNG == $ExtStr)) {
			if(file_exists($ImageFile)) {
				$this->OutputHeader($ExtStr);
				if(@readfile($ImageFile)) {
					return true;
				}
			}
			else {
				$this->errormessage = "画像ファイルが見つかりません";
			}
		}
		else {
			$this->errormessage = "画像タイプ指定が不正です";
		}
		return false;
	}

	/**
	 * 画像カレンダーファイル保存
	 *
	 * @param string $File			保存ファイル名
	 * @return true / false : 保存成功 / 保存失敗
	 */
	function Save($File = NULL) {
		if(!is_null($File)) {
			$OutFile = $File;
		}
		else if(isset($this->viewdata['imgfile'])) {
			$OutFile = $this->viewdata['imgfile'];
		}
		else {
			$this->errormessage = "保存先が指定されていません";
			return false;
		}
		if(YSC_CalendarUtil::filelockstatus($OutFile)) {
			$this->errormessage = "画像ファイルがロックされています";
			return false;
		}
		else {
			if(YSC_CalendarUtil::filelock($OutFile)) {
				if(!$this->OutputImage($OutFile)) {
					YSC_CalendarUtil::fileunlock($OutFile);
					$this->errormessage = "画像を保存できませんでした";
					return false;
				}
				YSC_CalendarUtil::fileunlock($OutFile);
			}
			else {
				$this->errormessage = "画像ファイルがロックされています";
				return false;
			}
		}
		return true;
	}

	/**
	 * 画像カレンダー出力
	 *
	 * @param string $File			保存ファイル名 (NULLの場合は、標準出力)
	 * @return true / false : 保存成功 / 保存失敗
	 */
	function OutputImage($File = NULL) {
		if(!isset($this->image)){
			if(!$this->MakeImage()) {
				$this->errormessage = "画像データの作成に失敗しました";
				return false;
			}
		}
		switch ($this->imgformat) {
			case CAL_IMG_TYPE_GIF :
				if($this->gd_info['GIF Create Support']){
					if(is_null($File)) {
						return imagegif($this->image);
					}
					else {
						return imagegif($this->image, $File);
					}
				}
				break;
			case CAL_IMG_TYPE_JPG :
			case CAL_IMG_TYPE_JPEG :
				if($this->gd_info['JPG Support']){
					if(is_null($File)) {
						return imagejpeg($this->image);
					}
					else {
						return imagejpeg($this->image, $File);
					}
				}
				break;
			case CAL_IMG_TYPE_PNG :
				if($this->gd_info['PNG Support']){
					if(is_null($File)) {
						return  imagepng($this->image);
					}
					else {
						return  imagepng($this->image, $File);
					}
				}
				break;
			default :
				$this->errormessage = "画像タイプ指定が不正です。";
				break;
		}
		return false;
	}

	/**
	 * ヘッダー出力
	 *
	 * @param string $Type		画像タイプ
	 * 							CAL_IMG_TYPE_JPEG,CAL_IMG_TYPE_JPG
	 * 							CAL_IMG_TYPE_GIF, CAL_IMG_TYPE_PNG
	 */
	function OutputHeader($Type) {
		switch ($Type) {
			case CAL_IMG_TYPE_GIF :
				header('Content-Type: image/gif');
				break;
			case CAL_IMG_TYPE_JPG :
			case CAL_IMG_TYPE_JPEG :
				header('Content-Type: image/jpeg');
				break;
			case CAL_IMG_TYPE_PNG :
				header('Content-Type: image/png');
				break;
			default :
				break;
		}
	}

	/**
	 * カレンダーイメージの作成
	 *
	 * @return true / false : 作成成功 / 作成失敗
	 */
	function MakeImage() {
		if(!isset($this->viewdata)) {
			return false;
		}
		// イメージ作成
		if(DISPLAY_CREDIT) {
			$Height = $this->viewdata['imgheight'] + 14;
		}
		else {
			$Height = $this->viewdata['imgheight'];
		}
		$this->image = imagecreate($this->viewdata['imgwidth'], $Height);
		if(!$this->image) {
			return false;
		}
		$Ret = true;
		$Color = $this->MakeColor($this->image, $this->viewdata['bordercolor']);
		if(!imagefill($this->image, 0, 0, $Color)) {
			$Ret = false;
		}
		foreach($this->viewdata['cells'] as $Cell) {
			// 背景カラー取得
			$Color = $this->MakeColor($this->image, $Cell['bgcolor']);
			// 背景描画
			 if(!imagefilledrectangle($this->image,
									  $Cell['position']['x1'],
									  $Cell['position']['y1'],
									  $Cell['position']['x2'],
									  $Cell['position']['y2'],
									  $Color )) {
				$Ret = false;
				break;
			}
			$Yplase = 0;
			foreach($Cell['text'] as $Text){
				if(('' != $Text['value']) && ('' != $Text['fontfile'])){
					$Tbox = imagettfbbox((float)$Text['fontsize'], 0, $Text['fontfile'], 'あ');
					$Yplase += $Tbox[7] * -1;
					$X = $Text['x1'] + $Tbox[0];
					$Y = $Text['y1'] + $Yplase;
					$Color = $this->MakeColor($this->image, $Text['fontcolor']);
					if(!imagettftext($this->image,(float)$Text['fontsize'], 0.0,
									 $X, $Y, $Color,
									 $Text['fontfile'], $Text['value'])) {
						$Ret = false;
					}
				}
			}
		}
		if(DISPLAY_CREDIT) {
			$Color = $this->MakeColor($this->image, 'FFFFFF');
			if(!imagefilledrectangle($this->image,
									 0,
									 $Height - 14,
									 $this->viewdata['imgwidth'] - 1,
									 $Height - 1,
									 $Color )) {
				$Ret = false;
			}
			$Color = $this->MakeColor($this->image, '000000');
			if(!imagettftext($this->image,(float)8, 0.0, 0, $Height - 2, $Color,
							 $this->viewdata['cells'][0]['text'][0]['fontfile'], PROG_CREDIT_IMG)) {
				$Ret = false;
			}
		}
		return $Ret;
	}

	/**
	 * HEXカラー値をイメージカラーインデック値に変換する
	 *
	 * @return !false / false : カラーインデックス値 / 変換失敗
	 */

	function MakeColor($Img, $Hexcolor){
		if(!$Hexcolor || !preg_match(REG_HEXCOLOR, $Hexcolor))
			return false;

		$Red   = hexdec(substr($Hexcolor, 0, 2));
		$Green = hexdec(substr($Hexcolor, 2, 2));
		$Blue  = hexdec(substr($Hexcolor, 4, 2));
		$Color = imagecolorallocate($Img, $Red, $Green, $Blue);
		return $Color;
	}
}
?>